#!/bin/bash

RETRY=5
SLEEP=20

WEBEX_LAMBDA="https://voxyzbifp6.execute-api.us-east-1.amazonaws.com/prod1"
IP_V4=`curl -s  http://169.254.169.254/latest/meta-data/public-ipv4`
START_TIME=`date +%s`
SCRIPT_DIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"

$SCRIPT_DIR/prometheus-metric.sh &
PROMETHEUS_PUSH_PID=$!

echo "Pushgateway PID=$PROMETHEUS_PUSH_PID"

function log () {
    LOG_TIMESTAMP=`date +"%Y-%m-%d-%T"`
    echo -e "$LOG_TIMESTAMP\t$1\t$2"
}
function create_backup_list(){
    find $CASSANDRA_DATA_DIR -type f | grep "/backups/" | grep -v "system-compactions_in_progress" | grep -v "tmp" | grep -v $TEMP_DIRNAME > $LIST_PATH
}

if [ -z "$CASSANDRA_DATA_DIR" ]
then
    log "ERROR" "CASSANDRA_DATA_DIR is EMPTY - can't continue"
    exit 1
fi

FILE_TIMESTAMP=`date +"%Y-%m-%d-%T" | sed 's#:#-#g'`
FILENAME="cassandra_backup_incremental_${FILE_TIMESTAMP}_${IP_V4}.tar"
DIRNAME="$TEMP_DATA_DIR/incremental_temp"
TEMP_DIRNAME="$DIRNAME/incremental"
ARCHIVE_PATH="${DIRNAME}/${FILENAME}"
LIST_PATH="${DIRNAME}/list.txt"
mkdir -p $DIRNAME
rm -rf $DIRNAME/*
mkdir -p $TEMP_DIRNAME


function copy_relevant_to_temp(){
    RELEVANT_LIST=$LIST_PATH
    DESTINATION=$TEMP_DIRNAME
    cat $RELEVANT_LIST | while read SINGLE
    do
        DIR1=$(dirname $SINGLE)
        DIR2=$(dirname $DIR1)
        TABLE_DIR=$(dirname $DIR2)
        TABLE_NAME=$(basename $DIR2)
        KEYSPACE_NAME=$(basename $TABLE_DIR)
        
        FILE_DEST=$DESTINATION/$DIR1
        
        mkdir -p $FILE_DEST
        if [ $? -ne 0 ]; then return 1; fi
        ln -v -s $SINGLE $FILE_DEST
        if [ $? -ne 0 ]; then return 1; fi
        
    done
    if [ $? -ne 0 ]; then return 1; fi
    return 0
}

function main_operation(){
    
    CURRENT_TRY=$1
    
    log "INFO" "Before start - empty!"
    rm -frv $DIRNAME/*
    if [ $? -ne 0 ]; then return 1; fi
    
    log "INFO" "Creating archive ${ARCHIVE_PATH}"
    
    create_backup_list
    FILE_COUNT=`cat ${LIST_PATH} | wc -l`
    
    if [[ "$FILE_COUNT" -eq 0 ]]
    then
        
        echo -e "\e[33mNo incremental files to backup\e[0m"
        return 0
    fi
    
    copy_relevant_to_temp $LIST_PATH
    if [ $? -ne 0 ]; then return 1; fi
    
    S3_PATH=s3://${S3_BUCKET}/$FILENAME
    EXPECTED_SIZE=`du -sL $TEMP_DIRNAME | cut -f1`
    EXPECTED_SIZE_HUMAN=`du -sLh $TEMP_DIRNAME | cut -f1`
    echo "Expected size = $EXPECTED_SIZE ($EXPECTED_SIZE_HUMAN)"
        
    tar chf - $TEMP_DIRNAME  | split -d -a 6 -b 2048M --filter 'aws s3 cp - s3://'"$S3_BUCKET"'/incremental-'"$FILE_TIMESTAMP"'-'"$IP_V4"'/'"$FILENAME"'.part$FILE' - 
    if [ $? -ne 0 ]; then return 1; fi
    
    log "INFO" "Done streaming archive!"
    
    BACKUP_SIZE=$EXPECTED_SIZE_HUMAN
    
    log "INFO" "Deleting incremental hardlinks!"
    cat $LIST_PATH | xargs rm -fv
    if [ $? -ne 0 ]; then return 1; fi
    log "INFO" "Deleting temp contents!"
    rm -frv $DIRNAME/*
    if [ $? -ne 0 ]; then return 1; fi

    
    END_TIME=`date +%s`
    DURATION=$((END_TIME - START_TIME))
    
    echo "cassandra_incremental_size_bytes $EXPECTED_SIZE" | curl --data-binary @- http://localhost:9092/metrics/job/cassandra_backup_incremental
    kill -9 $PROMETHEUS_PUSH_PID
    echo "cassandra_incremental_running_state 0" | curl --data-binary @- http://localhost:9092/metrics/job/cassandra_backup_incremental
    echo "cassandra_incremental_running_duration_seconds $DURATION" | curl --data-binary @- http://localhost:9092/metrics/job/cassandra_backup_incremental
    echo "cassandra_incremental_running_failed 0" | curl --data-binary @- http://localhost:9092/metrics/job/cassandra_backup_incremental

    # DURATION_STRING=`date -d@$DURATION -u +%H:%M:%S`
    # ATTEMPT_STRING="Size: $BACKUP_SIZE Attempt: ($CURRENT_TRY/$RETRY) Duration: $DURATION_STRING"
    # JSON="{\"outcome\":\"SUCCESS\",\"type\": \"${BACKUP_TYPE}\",\"ip\": \"${IP_V4}\",\"cluster\":\"${CLUSTER}\",\"message\":\"${ATTEMPT_STRING}\"}"
    # RES=`curl -i -X POST -H 'Content-Type: application/json' -d "$JSON" $WEBEX_LAMBDA`
    
    return 0
    
}

for ITER in $(seq 1 1 $RETRY)
do
    echo "Retry $ITER/$RETRY"
    
    main_operation $ITER
    RET=$?
    
    if [[ "$RET" -eq 0 ]]
    then
        echo "Exiting after success..."
        exit 0
    fi
    
    LAST_5=`journalctl -u cassandra-incremental -n 10 --no-pager`
    echo "Last error"
    echo $LAST_5
    
    MESSAGE=$(echo "${LAST_5//$'\n'/\\n}")
    
    JSON="{\"outcome\":\"FAILURE\",\"type\": \"${BACKUP_TYPE}\",\"ip\": \"${IP_V4}\",\"cluster\":\"${CLUSTER}\",\"message\":\"$MESSAGE\",\"ssh\":\"ssh -i ${IDENTITY_FILE} ${USER}@${IP_V4}\"}"
    RES=`curl -i -X POST -H 'Content-Type: application/json' -d "$JSON" $WEBEX_LAMBDA`
    
    echo -e "Sleeping for $SLEEP seconds..."
    sleep $SLEEP
done
echo "cassandra_incremental_running_state 0" | curl --data-binary @- http://localhost:9092/metrics/job/cassandra_backup_incremental
echo "cassandra_incremental_running_failed 1" | curl --data-binary @- http://localhost:9092/metrics/job/cassandra_backup_incremental
kill -9 $PROMETHEUS_PUSH_PID
exit 0