#!/bin/bash 

set -e

while getopts "hu:i:a:b:d:c:l:" opt; do
    case $opt in
        h)
            echo "Cassandra Incremental Backup Service Installer"
            echo -e "install.sh"
            echo -e "\t-u USER"
            echo -e "\t-i PEM file location"
            echo -e "\t-a host IP"
            echo -e "\t-b AWS S3 Bucket"
            echo -e "\t-c Cluster identificator"
            echo -e "\t-l Temporary directory location (/data/mount?) - will FAIL if not exists"
            echo -e "\t-d Cassandra data directory - will FAIL if not exists"
            exit 0
        ;;
        u) USER="$OPTARG"
        ;;
        i) PEM_LOCATION="$OPTARG"
        ;;
        a) IP_ADDRESS="$OPTARG"
        ;;
        b) S3_BUCKET="$OPTARG"
        ;;
        d) C_DATA_DIR="$OPTARG"
        ;;
        c) CLUSTER="$OPTARG"
        ;;
        l) TEMP_DIR="$OPTARG"
        ;;
        \?) echo "Invalid option -$OPTARG" >&2
            ERROR=1
        ;;
    esac
done

echo -e "\n"

if [ -z "$USER" ]
then
      echo -e "User is not defined"
      ERROR=1
fi
if [ -z "$PEM_LOCATION" ]
then
      echo -e "PEM file location is not defined"
      ERROR=1
fi
if [ -z "$IP_ADDRESS" ]
then
      echo -e "IP address is not defined"
      ERROR=1
fi
if [ -z "$C_DATA_DIR" ]
then
      echo -e "Cassandra data directory is not defined"
      ERROR=1
fi
if [ -z "$CLUSTER" ]
then
      echo -e "Cluster identificator is not defined"
      ERROR=1
fi
if [ -z "$TEMP_DIR" ]
then
      echo -e "Temporary directory location is not defined"
      ERROR=1
fi
if [[ "$ERROR" -eq 1 ]]
then

      echo -e "\n"
      echo -e "\e[33mExecute 'install.sh -h' for help \e[0m"
      echo -e "\e[31mExiting due to missing parameters \e[0m"
      exit 1
fi


echo -e "\e[7m   Please check the input   \e[39m\e[0m"
echo -e "\n"
echo -e "\e[33mUsing \e[32m${USER}@${IP_ADDRESS}\e[39m"
echo -e "\e[33mAuthenticating with \e[32m${PEM_LOCATION}\e[39m"
echo -e "\e[33mS3 bucket destination is \e[32m${S3_BUCKET} \e[0m"
echo -e "\e[33mCassandra data directory is \e[32m${C_DATA_DIR}\e[39m"
echo -e "\n"



remote () {
    ssh -i ${PEM_LOCATION} ${USER}@${IP_ADDRESS} "$1"
}

if ! ssh -i"${PEM_LOCATION}" "${USER}@${IP_ADDRESS}" "ls ${C_DATA_DIR} >/dev/null 2>&1" ; then
    echo "Cassandra data directory ${C_DATA_DIR} does not exist. Exiting."
    exit 1
fi
if ! ssh -i"${PEM_LOCATION}" "${USER}@${IP_ADDRESS}" "ls ${TEMP_DIR} >/dev/null 2>&1" ; then
    echo "Temp directory ${TEMP_DIR} does not exist. Exiting."
    exit 1
fi

echo -e "\e[34mInstalling  \e[91mAWS CLI\e[39m"
remote "sudo apt-get update && sudo DEBIAN_FRONTEND=noninteractive apt-get -y install unzip pigz parallel pv && env_parallel --install && echo  | parallel --will-cite echo"
set +e
remote  "cd ~ && curl \"https://awscli.amazonaws.com/awscli-exe-linux-x86_64.zip\" -o \"awscliv2.zip\" && unzip  -qq -o awscliv2.zip && sudo ./aws/install"
RES=$?

if [[ "$RES" -eq 1 ]]
then
    echo -e "Updating install"
    remote "sudo ./aws/install --update"    
fi

set -e

echo -e "\e[32mDone!\e[39m"
echo -e "\e[34mChecking  \e[91mAWS S3 access\e[39m"
NUM_BUCKETS=`remote "aws s3 ls | grep -Po "${S3_BUCKET}$" | wc -l"`
echo -e "found \e[41m  ${NUM_BUCKETS}  \e[0m buckets"

if [[ "$NUM_BUCKETS" -eq 0 ]]
then

      echo -e "\n"
      echo -e "\e[33mNone buckets found containing '${S3_BUCKET}' \e[0m"
      echo -e "\e[31mExiting due tofailed check \e[0m"
      exit 1
fi

IDENTITY_FILE=$(basename $PEM_LOCATION)

echo -e "\e[32mDone!\e[39m"
echo -e "\e[34mCopying service to \e[91m${IP_ADDRESS}\e[39m"
scp -r -i"${PEM_LOCATION}" cassandra-incremental-service "${USER}@${IP_ADDRESS}:~"
remote  "sudo rm -rf /opt/cassandra_backup_service"
remote  "sudo rm -rf /opt/cassandra-incremental-service"

remote  "sudo systemctl disable cassandra-backup.timer"

remote  "sudo mv ~/cassandra-incremental-service /opt"
remote  "sudo chown root:root -R /opt/cassandra-incremental-service"
echo -e "\e[32mDone!\e[39m"
echo -e "\e[34mContents of \e[91m/opt/cassandra-incremental-service directory\e[39m"
remote "ls -lha /opt/cassandra-incremental-service"
echo -e "\e[32mDone!\e[39m"
echo -e "\e[34mConfiguring service environment variables at \e[91m/opt/cassandra-incremental-service/env.sh\e[39m"
remote "sudo sed -e 's#REPLACE_WITH_S3_BUCKET#${S3_BUCKET}#' -i /opt/cassandra-incremental-service/env.sh"
remote "sudo sed -e 's#REPLACE_WITH_CASSANDRA_DATA_DIR#${C_DATA_DIR}#' -i /opt/cassandra-incremental-service/env.sh"
remote "sudo sed -e 's#REPLACE_WITH_CLUSTER#${CLUSTER}#' -i /opt/cassandra-incremental-service/env.sh"
remote "sudo sed -e 's#REPLACE_WITH_TEMP_DATA_DIR#${TEMP_DIR}#' -i /opt/cassandra-incremental-service/env.sh"
remote "sudo sed -e 's#REPLACE_WITH_IDENTITY#${IDENTITY_FILE}#' -i /opt/cassandra-incremental-service/env.sh"
remote "sudo sed -e 's#REPLACE_WITH_USER#${USER}#' -i /opt/cassandra-incremental-service/env.sh"
remote "sudo cat /opt/cassandra-incremental-service/env.sh"
echo -e "\e[32mDone!\e[39m"
echo -e "\e[34mEnabling services \e[91mcassandra-incremental.service \e[34mand \e[91mcassandra-incremental.timer\e[39m"
remote  "sudo cp -v /opt/cassandra-incremental-service/cassandra-incremental.service /etc/systemd/system/"
remote  "sudo cp -v /opt/cassandra-incremental-service/cassandra-incremental.timer /etc/systemd/system/"
remote  "sudo systemctl enable cassandra-incremental.service"
remote  "sudo systemctl enable cassandra-incremental.timer"
remote  "sudo systemctl stop cassandra-incremental.timer"
remote  "sudo systemctl daemon-reload"
remote  "sudo systemctl reset-failed"
remote  "sudo systemctl start cassandra-incremental.timer"
echo -e "\e[32mDone!\e[39m"
echo -e "\n"

 