#!/bin/bash

which kubectl > /dev/null 2>&1
if [ $? -ne 0 ] ; then
    echo "kubectl not found in path"
    exit 1
fi

# kubectl get pod | grep gitlab > /dev/null 2>&1
# if [ $? -eq 0 ] ; then
#     echo "Microservices already deployed"
#     exit 1
# fi

TOTAL_MEM=$(cat /proc/meminfo | grep MemTotal | awk '{print $2}')
MIN_MS_MEM_LIMIT=${MIN_MS_MEM_LIMIT:-14000000}
if [ $TOTAL_MEM -lt $MIN_MS_MEM_LIMIT ] ; then
    echo "You have insufficient memory to deploy the microservices feature"
    exit 1
fi

cd `dirname $0`

GL_TIMEOUT=900
GL_SUCCESS=`expr $GL_TIMEOUT + 100`
JENKINS_VER="2.70"

TIMEOUT_IN_MINUTES=30
TIMEOUT_IN_SECONDS=`expr $TIMEOUT_IN_MINUTES \* 60`
TIMEOUT_SLEEP_TIME=3
TIMEOUT_IN_LOOPS=`expr $TIMEOUT_IN_SECONDS / $TIMEOUT_SLEEP_TIME`
NOT_TIMEOUT=`expr $TIMEOUT_IN_LOOPS + 100`
PASS_THRESHOLD=30


function waitForGitlab {
	echo "Waiting for gitlab..."
	cnt=0
	pass_cnt=0
	while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
	do
		kubectl exec $GITLAB_POD -- nc -zv localhost 8080 > /dev/null 2>&1
	    if [ $? -eq 0 ] ; then
	        pass_cnt=`expr $pass_cnt + 1`
	        printf "-"
	        if [ $pass_cnt -ge $PASS_THRESHOLD ] ; then
	            cnt=`expr $NOT_TIMEOUT - 1`
	        fi
	    else
	        #Reset pass counter
	        pass_cnt=0
	        printf "!"
	    fi
	    sleep $TIMEOUT_SLEEP_TIME
	    cnt=`expr $cnt + 1`
	done
	echo

	if [ $cnt -ne $NOT_TIMEOUT ] ; then
		echo "TIMEOUT! gitlab has not come up in $TIMEOUT_IN_SECONDS seconds"
		exit 1
	fi
}

function waitForKubernetes {
	echo "Waiting for kubernetes..."
	cnt=0
	pass_cnt=0
    columns=NAMESPACE:.metadata.namespace,NAME:.metadata.name,REPLICAS:.status.replicas,AVAILABLE:.status.availableReplicas,LABELS:.metadata.labels
	while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
	do
		kubectl get deployment --all-namespaces -o custom-columns=$columns | tail -n+2 | grep -v 'connecthing.io/microservice' | awk '{print $3-$4}' | grep -v 0  > /dev/null 2>&1 
	    if [ $? -ne 0 ] ; then
	        pass_cnt=`expr $pass_cnt + 1`
	        printf "-"
	        if [ $pass_cnt -ge $PASS_THRESHOLD ] ; then
	            cnt=`expr $NOT_TIMEOUT - 1`
	        fi
	    else
	        #echo "Resetting pass counter to zero"
	        pass_cnt=0
	        printf "!"
	    fi
	    sleep $TIMEOUT_SLEEP_TIME
	    cnt=`expr $cnt + 1`
	done
	echo

	if [ $cnt -ne $NOT_TIMEOUT ] ; then
		echo "TIMEOUT! Kubernetes is not working properly, examine journalctl -u kubelet for more info"
		exit 1
	fi
}

function waitForK8SAPI {
	echo "Waiting for kubernetes API..."
	cnt=0
	while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
	do
		kubectl get ns  > /dev/null 2>&1 
	    if [ $? -eq 0 ] ; then
	        cnt=`expr $NOT_TIMEOUT - 1`
	    fi
	    sleep $TIMEOUT_SLEEP_TIME
	    printf "+"
	    cnt=`expr $cnt + 1`
	done
	echo

	if [ $cnt -ne $NOT_TIMEOUT ] ; then
		echo "TIMEOUT! k8s api not responding, Kubernetes is not working properly, examine journalctl -u kubelet for more info"
		exit 1
	fi
}

function waitForJenkins {
	echo "Waiting for jenkins..."
	cnt=0
	while [ $cnt -lt $GL_TIMEOUT ]
	do
		kubectl exec $JENKINS_POD -- nc -zv localhost 8080 > /dev/null 2>&1
	    if [ $? -eq 0 ] ; then
	        cnt=`expr $GL_TIMEOUT + 99`
	    fi
	    sleep 1
	    printf "-"
	    cnt=`expr $cnt + 1`
	done
	echo

	if [ $cnt -ne $GL_SUCCESS ] ; then
		echo "TIMEOUT! jenkins has not come up in $GL_TIMEOUT seconds"
		exit 1
	fi
}

bash gitlab/deploy.sh
bash jenkins/deploy.sh
bash docker-registry/deploy.sh


############# Gitlab ###########################################################

GITLAB_POD=$(kubectl get pod -n default | grep gitlab | awk '{print $1}')

waitForGitlab

kubectl cp gitlab/get_token.sh $GITLAB_POD:/get_token.sh
GITLAB_TOKEN=`kubectl exec $GITLAB_POD -- bash /get_token.sh`
echo $GITLAB_TOKEN

GITLAB_TOKEN_BASE64=`echo -n $GITLAB_TOKEN | base64`
echo $GITLAB_TOKEN_BASE64

sed s/GITLAB_TOKEN64/$GITLAB_TOKEN_BASE64/g gitlab/gitlab-secret.yaml.tpl  > gitlab-secret.yaml
kubectl replace -f gitlab-secret.yaml


################# Jenkins #####################################################

JENKINS_POD=$(kubectl get pod -n default | grep jenkins | awk '{print $1}')

waitForJenkins

curl -O https://s3.amazonaws.com/downloads.davra.com/jenkins-state/jenkins-state-$JENKINS_VER.tar.gz
#kubectl cp  jenkins-state-$JENKINS_VER.tar.gz $JENKINS_POD:/jenkins-state-$JENKINS_VER.tar.gz
#kubectl exec $JENKINS_POD -- tar -zxf /jenkins-state-$JENKINS_VER.tar.gz -C /var/jenkins_home --strip-components=1 
# NOTE: this will now only work on on-prem installs
tar -zxf jenkins-state-$JENKINS_VER.tar.gz -C /var/Davra/
kubectl exec $JENKINS_POD -- pkill java

waitForJenkins

################### Enable microservices feature ##############################
MS_MGR_POD=$(kubectl get pod | grep ruban-ms-microservice-manager | awk '{print $1}')

kubectl cp enable-microservices.js $MS_MGR_POD:/ruban-ms-microservice-manager/enable-microservices.js
kubectl exec $MS_MGR_POD -- node /ruban-ms-microservice-manager/enable-microservices.js
kubectl exec $MS_MGR_POD -- pkill node

################### Docker Registry ###########################################
DOCKER_REGISTRY=$(kubectl get service docker-registry -o jsonpath='{.spec.clusterIP}')
echo "Docker registry: $DOCKER_REGISTRY"
sed s/DOCKER_REGISTRY_IP/$DOCKER_REGISTRY/g docker-registry/daemon.json > daemon.json
mkdir -p /etc/docker
cp daemon.json /etc/docker/

NODE=`kubectl get node | tail -n+2 | awk '{print $1}'`
kubectl drain $NODE --force --delete-local-data --ignore-daemonsets
systemctl stop kubelet
systemctl restart docker
systemctl start kubelet
waitForK8SAPI
kubectl uncordon $NODE
waitForKubernetes

#Wait again for Gitlab to come up after restarting everything 
GITLAB_POD=$(kubectl get pod -n default | grep gitlab | awk '{print $1}')
waitForGitlab

