#!/bin/bash
#
# To be run after `ruban-ms-backups/restore-from-host.sh`
# This will ensure the correct gitlab-secret is being used,
# and that the gitlab directories are accessible.
#

which kubectl > /dev/null 2>&1
if [ $? -ne 0 ] ; then
    curl -LO https://dl.k8s.io/release/v1.19.0/bin/linux/amd64/kubectl
    chmod +x kubectl
    PATH=$PATH:$(pwd)
fi

which kubectl > /dev/null 2>&1
if [ $? -ne 0 ] ; then
    echo "kubectl not found in path" 1>&2
    exit 1
fi

#root only
if [ "$(id -u)" != "0" ]; then
    echo "This script must be run as root" 1>&2
    exit 1
fi

cd `dirname $0`

TIMEOUT_IN_MINUTES=30
TIMEOUT_IN_SECONDS=`expr $TIMEOUT_IN_MINUTES \* 60`
TIMEOUT_SLEEP_TIME=3
TIMEOUT_IN_LOOPS=`expr $TIMEOUT_IN_SECONDS / $TIMEOUT_SLEEP_TIME`
NOT_TIMEOUT=`expr $TIMEOUT_IN_LOOPS + 100`
PASS_THRESHOLD=30

function waitForGitlab {
    echo "Waiting for gitlab..."
    cnt=0
    pass_cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl exec $GITLAB_POD -- nc -zv localhost 8080 > /dev/null 2>&1
        if [ $? -eq 0 ] ; then
            pass_cnt=`expr $pass_cnt + 1`
            printf "-"
            if [ $pass_cnt -ge $PASS_THRESHOLD ] ; then
                cnt=`expr $NOT_TIMEOUT - 1`
            fi
        else
            #Reset pass counter
            pass_cnt=0
            printf "!"
        fi
        sleep $TIMEOUT_SLEEP_TIME
        cnt=`expr $cnt + 1`
    done
    echo
    
    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! gitlab has not come up in $TIMEOUT_IN_SECONDS seconds"
        exit 1
    fi
}


function waitForKubernetes {
    echo "Waiting for kubernetes..."
    cnt=0
    pass_cnt=0
    columns=NAMESPACE:.metadata.namespace,NAME:.metadata.name,REPLICAS:.status.replicas,AVAILABLE:.status.availableReplicas,LABELS:.metadata.labels
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl get deployment --all-namespaces -o custom-columns=$columns | tail -n+2 | grep -v 'connecthing.io/microservice' | awk '{print $3-$4}' | grep -v 0  > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            pass_cnt=`expr $pass_cnt + 1`
            printf "-"
            if [ $pass_cnt -ge $PASS_THRESHOLD ] ; then
                cnt=`expr $NOT_TIMEOUT - 1`
            fi
        else
            #echo "Resetting pass counter to zero"
            pass_cnt=0
            printf "!"
        fi
        sleep $TIMEOUT_SLEEP_TIME
        cnt=`expr $cnt + 1`
    done
    echo
    
    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! Kubernetes is not working properly, examine journalctl -u kubelet for more info"
        exit 1
    fi
}

function waitForK8SAPI {
    echo "Waiting for kubernetes API..."
    cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl get ns  > /dev/null 2>&1
        if [ $? -eq 0 ] ; then
            cnt=`expr $NOT_TIMEOUT - 1`
        fi
        sleep $TIMEOUT_SLEEP_TIME
        printf "+"
        cnt=`expr $cnt + 1`
    done
    echo
    
    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! k8s api not responding, Kubernetes is not working properly, examine journalctl -u kubelet for more info"
        exit 1
    fi
}

# Get Gitlab pod, then get token, then update k8s secret

GITLAB_POD=$(kubectl get pod -n default | grep gitlab | awk '{print $1}')

waitForGitlab

kubectl cp gitlab/get_token.sh $GITLAB_POD:/get_token.sh
GITLAB_TOKEN=`kubectl exec $GITLAB_POD -- bash /get_token.sh`
echo $GITLAB_TOKEN

GITLAB_TOKEN_BASE64=`echo -n $GITLAB_TOKEN | base64`
echo $GITLAB_TOKEN_BASE64

sed s/GITLAB_TOKEN64/$GITLAB_TOKEN_BASE64/g gitlab/gitlab-secret.yaml.tpl  > gitlab-secret.yaml
kubectl replace -f gitlab-secret.yaml

# Ensure dirs are readable
echo $SUDO_USER
chown -R $SUDO_USER:$SUDO_USER /var/Davra/gitlab


#
# Restart cluster after restore
#

NODE=`kubectl get node | tail -n+2 | awk '{print $1}'`
kubectl drain $NODE --force --delete-local-data --ignore-daemonsets
systemctl stop kubelet
systemctl restart docker
systemctl start kubelet
waitForK8SAPI

kubectl uncordon $NODE
waitForKubernetes

#Wait again for Gitlab to come up after restarting everything
GITLAB_POD=$(kubectl get pod -n default | grep gitlab | awk '{print $1}')
waitForGitlab

# Ensure dirs are readable again
echo $SUDO_USER
chown -R $SUDO_USER:$SUDO_USER /var/Davra/gitlab
