cd `dirname $0`

source ../utils.sh

# These PV setups happen here because on localhost we need to explicitly
# create the PV ourselves then wen can create the PVC. However on AWS we can 
# go straight to create the PVC and the PV will be created by kubernetes using AWS handlers.
doPV postgres
doPV mongodb
doPV zookeeper
doPV kafka-data
doPV ruban
doPV cassandra
doPV redis
doPV mqtt
doPV gitlab
doPV docker-registry
doPV microservice-workspaces

doPVC gitlab
doPV jenkins
doPVC jenkins
doPVC docker-registry

#pull docker images
ls -1 ../*deployment.yaml | awk '{system("grep -i image "$1)}' | awk '{print $NF}' | awk '{system("docker pull " $1)}'
docker pull docker.io/davradocker/jenkins-jnlp-build-slave:0.0.1

# Now create the PVCs . Any PVs will have been created on localhost already or on-demand for AWS
/bin/bash ../deploy-infrastructure-pvcs.sh
if [ $? -ne 0 ] ; then
	exit 1
fi

/bin/bash ../deploy-infrastructure.sh
if [ $? -ne 0 ] ; then
	exit 1
fi

#kafka subpath migration
if [ -d /var/Davra/kafka/data/kafka-data/kafka-data ] ; then
	echo "Handling kafka pv migration"
	rm -rf /var/Davra/kafka/data/kafka-data/kafka-data
fi

/bin/bash ../deploy-kafka-topics.sh
if [ $? -ne 0 ] ; then
	exit 1
fi

#Tell microservice legacy tenant controller what type of storage to give legacy tenants
doConfigmap microservice-manager-env

KAIROSDB_CONSISTENCY_READ=${KAIROSDB_CONSISTENCY_READ:-ONE}
KAIROSDB_CONSISTENCY_WRITE=${KAIROSDB_CONSISTENCY_WRITE:-ONE}
KAIROSDB_WRITE_BUFFER_MAX_SIZE=${KAIROSDB_WRITE_BUFFER_MAX_SIZE:-500000}
KAIROSDB_SINGLE_ROW_READ_SIZE=${KAIROSDB_SINGLE_ROW_READ_SIZE:-10240}
KAIROSDB_QUERY_CACHE_CACHE_FILE_CLEANER_SCHEDULE=${KAIROSDB_QUERY_CACHE_CACHE_FILE_CLEANER_SCHEDULE:-0 0 12 ? * SUN *}

cat ../kairosdb-config-configmap.yaml.tpl > kairosdb-config-configmap.yaml
sed -i s/KAIROSDB_CONSISTENCY_READ/$KAIROSDB_CONSISTENCY_READ/g kairosdb-config-configmap.yaml
sed -i s/KAIROSDB_CONSISTENCY_WRITE/$KAIROSDB_CONSISTENCY_WRITE/g kairosdb-config-configmap.yaml
sed -i s/KAIROSDB_WRITE_BUFFER_MAX_SIZE/$KAIROSDB_WRITE_BUFFER_MAX_SIZE/g kairosdb-config-configmap.yaml
sed -i s/KAIROSDB_SINGLE_ROW_READ_SIZE/$KAIROSDB_SINGLE_ROW_READ_SIZE/g kairosdb-config-configmap.yaml
sed -i "s/KAIROSDB_QUERY_CACHE_CACHE_FILE_CLEANER_SCHEDULE/$KAIROSDB_QUERY_CACHE_CACHE_FILE_CLEANER_SCHEDULE/g" kairosdb-config-configmap.yaml

doConfigmap kairosdb-config

/bin/bash ../deploy-services.sh
if [ $? -ne 0 ] ; then
	exit 1
fi

#ingress controller
kubectl get deployment default-http-backend -n kube-system > /dev/null 2>&1 
if [ $? -ne 0 ] ; then
	kubectl create -f ../infrastructure/default-backend-deployment.yaml
fi

kubectl get configmap nginx-ingress-controller -n kube-system > /dev/null 2>&1 
if [ $? -ne 0 ] ; then
	kubectl create -f ../infrastructure/nginx-ingress-controller-configmap.yaml
else
	kubectl replace -f ../infrastructure/nginx-ingress-controller-configmap.yaml
fi

#Metrics server
kubectl apply -f ../infrastructure/metrics-server/metrics-server.yaml

# Must use doDeployment to preserve any resource changes
CURR_DIR=`pwd`
cd ../infrastructure
kubectl apply -f nginx-permissions.yaml
kubectl scale deployment nginx-ingress-controller -n kube-system --replicas=0

# Check if cert exists, so we can re-apply it
EXISTING_CERT=$(kubectl get deployment -n kube-system nginx-ingress-controller -o json | grep "default-ssl-certificate=default/host.tls.assets")

doDeployment nginx-ingress-controller kube-system
doService nginx-ingress-controller kube-system

if [[ -n "$EXISTING_CERT" ]]; then
    kubectl get deployment -n kube-system nginx-ingress-controller -o json | jq '.spec.template.spec.containers[0].args[3] |= . + "--default-ssl-certificate=default/host.tls.assets"' > .ingress-controller-update
    kubectl replace -f .ingress-controller-update
    rm .ingress-controller-update
fi

kubectl scale deployment nginx-ingress-controller -n kube-system --replicas=1
cd $CURR_DIR


# Manual gitlab/jenkins ingress - Delete if present
kubectl get ingress gitlab > /dev/null 2>&1 
if [ $? -eq 0 ] ; then
	echo "Deleting gitlab ingress"
	kubectl delete ingress gitlab
fi
kubectl get ingress jenkins > /dev/null 2>&1 
if [ $? -eq 0 ] ; then
	echo "Deleting jenkins ingress"
	kubectl delete ingress jenkins
fi

# Elastic search
kubectl get deployment elasticsearch > /dev/null 2>&1 
if [ $? -ne 0 ] ; then
    cd ../infrastructure/elasticsearch/onprem
    mkdir -p /var/Davra/elasticsearch/data
    chmod -R 777 /var/Davra/elasticsearch
    doPV elasticsearch
    doPVC elasticsearch
    doDeployment elasticsearch
    doService elasticsearch
    pwd
    cd $CURR_DIR
    pwd
fi

# Logging
kubectl get ns logging > /dev/null 2>&1 
if [ $? -eq 0 ] ; then
    echo "Logging already configured, skipping"
else
    export FLUENT_CONFIGMAP_TPL=${FLUENT_CONFIGMAP_TPL:-"fluent-bit-onprem-configmap.yaml.tpl"}
    export DAVRA_DEPLOYMENT=${FLUENT_DAVRA_DEPLOYMENT:-"ON_PREM"}
    if [$LOG_LEVEL == "QUIET"]; then
        bash ../infrastructure/logging/install.sh
    else
        bash -x ../infrastructure/logging/install.sh
    fi
    if [ $? -ne 0 ] ; then
        echo "Logging setup failed"
        exit 1
    fi

    if [[ -n "$ELASTICSEARCH_CREDENTIALS" ]] ; then
        echo "$ELASTICSEARCH_CREDENTIALS" | base64 -d > esconfig.json
        kubectl create secret generic escredentials  --from-file=esconfig.json
        rm esconfig.json
        cd ..
        doDeployment davra-logs-to-es-persist-service
        cd $CURR_DIR
    fi
fi

# Put the only node into the rules-engine group so rule-worker nodes can be scheduled onto it
kubectl label nodes --all group=rules-engine


if [$LOG_LEVEL == "QUIET"]; then
    /bin/bash ../deploy-settings.sh
else
    /bin/bash -x ../deploy-settings.sh
fi
if [ $? -ne 0 ] ; then
    echo "Failed to deploy settings"
    exit 1
fi

#Wait for the ruban deployment to come up
waitOnDeployment davra-api