#!/bin/bash

cd `dirname $0`

#root only
if [ "$(id -u)" != "0" ]; then
  echo "This script must be run as root" 1>&2
  exit 1
fi

export PATH=`pwd`:$PATH

source ../utils.sh

#Ensure loopback is in the hosts file
if [ ! -f "/etc/hosts" ]; then
  echo "127.0.0.1 localhost" > /etc/hosts
  else
    if ! grep -q localhost /etc/hosts  ; then
      echo "127.0.0.1 localhost" >> /etc/hosts
    fi
fi

#Configure rate limiting on journald
cp /etc/systemd/journald.conf /etc/systemd/journald.conf.`date +%s`
sed -i s/#RateLimitIntervalSec=30s/RateLimitIntervalSec=0/g /etc/systemd/journald.conf
systemctl restart systemd-journald

#Disable auto-update
./apply_coreos_settings.sh

#kubernetes
/bin/bash install_kubernetes.sh
if [ $? -ne 0 ] ; then
  echo "Installation of kubernetes failed exiting installation"
  exit 1
else
  echo "Kubernetes install complete"
fi

if which docker-compose  > /dev/null 2>&1 ; then
    touch /var/Davra/RuBAN/.migrated
fi

RUBAN_VER='none'
RUBAN_INSTALLED=false

if docker ps | grep ruban:6 > /dev/null
then
RUBAN_VER=`docker exec ruban cat /opt/Davra/RuBAN/ruban.version`
RUBAN_INSTALLED=true
echo "Installed RuBAN version $RUBAN_VER"
fi

if [ "$RUBAN_INSTALLED" = true ] ; then

 if [ ! -f /var/Davra/RuBAN/.migrated ] ; then
  echo "######################################################"
  echo "Starting migrating data for postgresql and mongodb ...."
  echo "######################################################"
   ./migratedata.sh
  else
   echo "...OK..."
  fi
fi
touch /var/Davra/RuBAN/.migrated

#Ensure cassandra data is in the right place
if [ -d "/var/Davra/cassandra/data/data" ]; then
   cp -r /var/Davra/cassandra/data/data /var/Davra/bkp
   rm -rf /var/Davra/cassandra/data/data
   cp -r /var/Davra/bkp/data/* /var/Davra/cassandra/data
fi

######################################################################
#Handle migration from docker-compose to kubernetes
docker ps | grep "ruban:" > /dev/null 2>&1 
if [ $? -eq 0 ] ; then
  echo "Removing old ruban containers"
	docker ps | tail -n +2 | awk '{system("docker stop "$1)}'
  docker ps -a | tail -n +2 | awk '{system("docker rm "$1)}'
  docker images | tail -n +2 | grep -v davradocker | grep -v quay | grep -v gcr.io | awk '{system("docker rmi " $3)}'
fi

#move application folder under /var/Davra/RuBAN
if [ ! -d /var/Davra/RuBAN/applications ] ; then
    if [ -d /var/Davra/applications ] ; then
    	mv /var/Davra/applications /var/Davra/RuBAN
    fi
fi

#migrate postgres data to align on-prem and openstack deployments
if [ ! -d /var/Davra/postgresql/data/pgdata ] ; then
    mv /var/Davra/postgresql/data /var/Davra/postgresql/pgdata
    mkdir /var/Davra/postgresql/data
    mv /var/Davra/postgresql/pgdata /var/Davra/postgresql/data/pgdata
fi

######################################################################


# Set UDP receive buffer sizes (for videomgr ffmpeg transcoding of higher resolutions)
if [ ! -f "/etc/sysctl.d/sysctl.conf" ] ; then
  echo "net.core.rmem_max=8388608" >> /etc/sysctl.d/sysctl.conf
  echo "net.core.rmem_default=1024000" >> /etc/sysctl.d/sysctl.conf
  sysctl -p /etc/sysctl.d/sysctl.conf
fi

# set vm.max_map_count for ElasticSearch
grep vm.max_map_count /etc/sysctl.d/sysctl.conf > /dev/null 2>&1 
if [ $? -ne 0 ] ; then
  echo "vm.max_map_count=262144" >> /etc/sysctl.d/sysctl.conf
  sysctl -p /etc/sysctl.d/sysctl.conf
fi


#Remove old nginx 'router'
docker ps -a -f name=nginx --format "{{.Names}}" | grep -x nginx > /dev/null 2>&1 
if [ $? -eq 0 ] ; then
  docker stop nginx
  docker rm -f nginx
fi

#ruban infrastructure: kafka + databases
mkdir -p /var/Davra/RuBAN
/bin/bash deploy.sh
if [ $? -ne 0 ] ; then
  exit 1
fi

iptables -t nat -L | grep REDIRECT | grep 58000 | grep "redir ports 80" > /dev/null 2>&1 
if [ $? -ne 0 ] ; then
  export PRIMARY_IFACE=`route |grep default | head -1 | awk '{print $NF}'`
  iptables -t nat -A PREROUTING -p tcp -i $PRIMARY_IFACE --dport 58000 -j REDIRECT --to-port 80
  iptables-save > /var/lib/iptables/rules-save
  systemctl enable iptables-restore 
fi

#tag the default namespace as the 'ruban' tenant
kubectl describe ns default | grep -A 4 Labels | grep tenantId
if [ $? -ne 0 ] ; then
	kubectl label namespace default tenantId=ruban
fi

cnt=0

function printWaitMessage {
    #loop waiting for deployments to complete...
    echo "####################################################################################################################"
    echo "Waiting for deployments to come up."
    echo "This may take a long time: 10-30 mins or longer depending on your internet connection. Don't stop the installation."
    echo "####################################################################################################################"
    echo ""
}

TIMEOUT_IN_MINUTES=30
TIMEOUT_IN_SECONDS=`expr $TIMEOUT_IN_MINUTES \* 60`
TIMEOUT_SLEEP_TIME=3
TIMEOUT_IN_LOOPS=`expr $TIMEOUT_IN_SECONDS / $TIMEOUT_SLEEP_TIME`
NOT_TIMEOUT=`expr $TIMEOUT_IN_LOOPS + 100`
PASS_THRESHOLD=10

#Deploy kubelet memory usage monitor
cp ../monitoring/kubelet-monitor/kubelet-check.sh /root
cp ../monitoring/kubelet-monitor/kubelet-monitor.service /etc/systemd/system/
cp ../monitoring/kubelet-monitor/kubelet-monitor.timer /etc/systemd/system/
systemctl enable kubelet-monitor.timer
systemctl start kubelet-monitor.timer

function waitForKubernetes {
	echo "Waiting for kubernetes..."
	cnt=0
	pass_cnt=0
	while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
	do
		#kubectl get ns > /dev/null 2>&1
		kubectl get deployment --all-namespaces --show-labels | tail -n+2 | grep -v 'connecthing.io/microservice' | awk '{print $3-$6}' | grep -v 0 > /dev/null 2>&1 
	    if [ $? -ne 0 ] ; then
	        pass_cnt=`expr $pass_cnt + 1`
	        if [ $pass_cnt -ge $PASS_THRESHOLD ] ; then
	            cnt=`expr $NOT_TIMEOUT - 1`
	        fi
	    else
	        #echo "Resetting pass counter to zero"
	        pass_cnt=0
	    fi
	    sleep $TIMEOUT_SLEEP_TIME
	    printf "-"
	    cnt=`expr $cnt + 1`
	done
	echo

	if [ $cnt -ne $NOT_TIMEOUT ] ; then
		echo "TIMEOUT! Kubernetes is not working properly, examine journalctl -u kubelet for more info"
		exit 1
	fi
	
	echo "$pass_cnt successive passes for deployment checks, all is well"
}

#Migrate licence
if [ -f /var/Davra/RuBAN/database/rubanData/license.lic ] ; then
    LICENCE_TXT=`cat /var/Davra/RuBAN/database/rubanData/license.lic | base64 -w 0`
    sed s/LICENCE/$LICENCE_TXT/g licence.json > .licence.json
    
    echo "Migrating licence from old ruban POD to davra-api"
    curl -f -X PUT -u root:root --data-binary "@.licence.json" -H "content-type: application/json" http://localhost/api/v1/licence/ruban
    if [ $? -eq 0 ] ; then
        mv /var/Davra/RuBAN/database/rubanData/license.lic /var/Davra/RuBAN/database/rubanData/license.lic.migrated
    fi
fi


if [ "T$DEPLOY_MICROSERVICES" != 'T' ] ; then
    echo "Deploying microservices"
    export PATH=$PATH:`pwd`
    bash ../infrastructure/install-microservice-infrastructure.sh
fi

printWaitMessage
waitForKubernetes
