#!/bin/bash
HOME=$(pwd)
cd $(dirname $0)

KUBE_VERSION=1.19.9-00
DOCKER_VERSION=18.06.3~ce~3-0~debian

POD_NETWORK=${POD_NETWORK:-"10.244.0.0"}
POD_NETWORK_MASK=${POD_NETWORK_MASK:-"16"}
POD_NETWORK_CIDR=${POD_NETWORK}/${POD_NETWORK_MASK}

SERVICE_NETWORK=${SERVICE_NETWORK:-"10.3.0.0"}
SERVICE_NETWORK_MASK=${SERVICE_NETWORK_MASK:-"24"}
SERVICE_NETWORK_CIDR=${SERVICE_NETWORK}/${SERVICE_NETWORK_MASK}

source ../utils.sh

# export existing config kubectl picks it up
EXISTING_KUBE_CONFIG=/etc/kubernetes/admin.conf
if [ -f "$EXISTING_KUBE_CONFIG" ]; then
    export KUBECONFIG=/etc/kubernetes/admin.conf
fi

########################################################## STAGE 1: PRE-INSTALL CHECKLIST
#
# CHECK 1: root only: must have sudo installed and be running as root
SUDO_INSTALLED=$(which sudo)
if [ -z "$SUDO_INSTALLED" ]; then
    SUDO_STATUS="sudo could not be found. Please run 'apt install sudo' as 'su' "
else
    if [ "$(id -u)" != "0" ]; then
        SUDO_STATUS="This script must be run as root"
    fi
fi

# CHECK 2: Determine OS, version and code name
OS_STATUS=""
if [ -s /etc/os-release ]; then
    . /etc/os-release
    if [ "$ID" != "debian" ]; then
        OS_STATUS="This script can only run on Debian"
    fi
    if [ "$VERSION_ID" -ne "10" ]; then
        OS_STATUS="This script can only run on Debian 10"
    fi
else
    # /etc/os-release required by/part of systemd, if not available....
    OS_STATUS="Can't find /etc/os-release. Is this a compatible platform?"
fi

# CHECK 3: if sbin not in path, set it:
HAS_SBIN=$(echo $PATH | grep sbin)
if [ -z $HAS_SBIN ]; then
    export PATH=/sbin:$PATH
    echo "export PATH=/sbin/:$PATH" >>/root/.bashrc
    echo "export PATH=/sbin/:$PATH" >>/home/$USER/.bashrc
fi

# CHECK 4: if iptables are not set to legacy, set them:
IPTABLES_LEGACY=$(echo iptables -V | grep legacy)
if [ -z $IPTABLES_LEGACY ]; then
    update-alternatives --set iptables /usr/sbin/iptables-legacy
    update-alternatives --set ip6tables /usr/sbin/ip6tables-legacy
    # update-alternatives --set arptables /usr/sbin/arptables-legacy
    # update-alternatives --set ebtables /usr/sbin/ebtables-legacy
fi

# CHECK 5: Swap Memory
SWAP_STATUS=""
SWAP_AMOUNT=$(free -g | grep Swap | awk '{print $2}')
if [ ! -z $SWAP_AMOUNT ] && [ $SWAP_AMOUNT -ne "0" ]; then
    SWAP_STATUS="Swap memory is still on. This must be disabled before installing."
fi

# CHECK 6: Free Memory
MEM_STATUS=""
FREE_MEM=$(free -g | grep Mem | awk '{print $2}')
REQ_MEM="16" # required RAM in GB
if [ -z $FREE_MEM ]; then
    MEM_STATUS="Could not detect available Memory. You should double check this."
else
    if [ $FREE_MEM -lt $REQ_MEM ]; then
        MEM_STATUS="Less than 16GB of Ram detected. This is below the recommended requirements."
    fi
fi

# CHECK 7: Free Disk Space
DISK_ERROR=""
DISK_WARNING=""
FREE_SPACE=$(df -k . | awk 'NR==2 { print $4 }')
# Minimum free disk space, 40GB == 41,943,040 - if less than this, then exit with error
MIN_SPACE=41000000 # Rounded down
# Least Recommended space, 60GB == 62,914,560, but round down to 62 million
REC_SPACE=62000000 # Less than this is a warning, but you can install for test anyway
if [ -z $FREE_SPACE ]; then
    DISK_WARNING="Could not detect available disk space. You should double check this."
else
    if [ $FREE_SPACE -lt $MIN_SPACE ]; then
        DISK_ERROR="Less than 40GB of free disk space detected. This is below the minimum requirements."
    else
        if [ $FREE_SPACE -lt $REC_SPACE ]; then
            DISK_WARNING="Less than 60GB of free disk space detected. This is below the recommended requirements."
        fi
    fi
fi

# CHECK 8: If k8s already installed? And what about the Davra platform?
KUBE_INSTALLED=$(which kubectl)
KUBE_EXISTING_VERSION=""
if [ ! -z "$KUBE_INSTALLED" ]; then
    KUBE_STATUS="kubectl already installed "
    DAVRA_STATUS=$(kubectl describe namespace -l tenantId=ruban | grep Name)
    if [ ! -z "$DAVRA_STATUS" ]; then
        KUBE_STATUS="kubectl already installed. Davra platform already installed."
    fi
    KUBE_EXISTING_VERSION=$(dpkg -s kubectl | grep -i version)
fi

# CHECK 9: Kernel flags
OVERCOMMIT_MEM_STATUS=""
OVERCOMMIT_MEM=$(sysctl vm.overcommit_memory | awk '{print $3}')
if [ ! -z $OVERCOMMIT_MEM ] && [ $OVERCOMMIT_MEM -ne "1" ]; then
    OVERCOMMIT_MEM_STATUS=$'Kernel flag vm.overcommit_memory needs to be set to 1.\nAdd the following line to /etc/sysctl.conf and reboot before retrying\n vm.overcommit_memory = 1'
fi
KPANIC_STATUS=""
KPANIC=$(sysctl kernel.panic | awk '{print $3}')
MIN_KPANIC="1"
if [ ! -z $KPANIC ] && [ $KPANIC -lt $MIN_KPANIC ]; then
    KPANIC_STATUS=$'Kernel flag kernel.panic cannot be 0. \nAdd the following line to /etc/sysctl.conf and reboot before retrying\n kernel.panic = 10'
fi
KPANIC_ON_OOPS_STATUS=""
KPANIC_ON_OOPS=$(sysctl kernel.panic_on_oops | awk '{print $3}')
if [ ! -z $KPANIC_ON_OOPS ] && [ $KPANIC_ON_OOPS -ne "1" ]; then
    KPANIC_ON_OOPS_STATUS=$'Kernel flag kernel.panic_on_oops needs to be set to 1.\nAdd the following line to /etc/sysctl.conf and reboot before retrying\n kernel.panic_on_oops = 1'
fi

DHCP_STATUS=$(cat /etc/network/interfaces | grep dhcp)

# Other enviroment vars?

#######################################################################################
# Prep screens:
# check if stdout is a terminal and supports colours:
if test -t 1; then
    IS_TERMINAL=yes
    ncolors=$(tput colors)
    if test -n "$ncolors" && test $ncolors -ge 8; then
        BOLD="$(tput bold)"
        UNDERLINE="$(tput smul)"
        STANDOUT="$(tput smso)"
        NORMAL="$(tput sgr0)"
        BLACK="$(tput setaf 0)"
        RED="$(tput setaf 1)"
        GREEN="$(tput setaf 2)"
        YELLOW="$(tput setaf 3)"
        BLUE="$(tput setaf 4)"
        MAGENTA="$(tput setaf 5)"
        CYAN="$(tput setaf 6)"
        WHITE="$(tput setaf 7)"
    fi
fi

HAS_ERRORS=""
HAS_WARNINGS=""
function printOK() {
    echo -e "${GREEN}[OK] $1 ${NORMAL}" 1>&2
}
function printWarning() {
    echo -e "${YELLOW}[WARN] $1 ${NORMAL}" 1>&2
    HAS_WARNINGS="true"
}
function printError() {
    echo -e "${RED}[ERROR] $1 ${NORMAL}" 1>&2
    HAS_ERRORS="true"
}

if [ ! -z "$IS_TERMINAL" ]; then
    export LOG_LEVEL="QUIET"

    echo -e "
██████╗  █████╗ ██╗   ██╗██████╗  █████╗
██╔══██╗██╔══██╗██║   ██║██╔══██╗██╔══██╗
██║  ██║███████║██║   ██║██████╔╝███████║
██║  ██║██╔══██║╚██╗ ██╔╝██╔══██╗██╔══██║
██████╔╝██║  ██║ ╚████╔╝ ██║  ██║██║  ██║
╚═════╝ ╚═╝  ╚═╝  ╚═══╝  ╚═╝  ╚═╝╚═╝  ╚═╝

"
    echo
    echo -e "${BOLD}${CYAN} NOTE: This installation can take up to an hour.${NORMAL}${NORMAL}" 1>&2
    echo -e "${BOLD}${CYAN} Once started, do not interrupt even when it may appear stuck.${NORMAL}${NORMAL}" 1>&2
    echo -e "${BOLD}${CYAN} The installation will exit eventually.${NORMAL}${NORMAL}" 1>&2
    echo

    # For each check, display status:
    if [ ! -z "$SUDO_STATUS" ]; then
        printError "$SUDO_STATUS"
    else
        printOK "sudo OK"
    fi

    if [ ! -z "$OS_STATUS" ]; then
        printError "$OS_STATUS"
    else
        printOK "System OS: $NAME $VERSION_CODENAME $VERSION_ID"
    fi

    if [ ! -z "$SWAP_STATUS" ]; then
        printError "$SWAP_STATUS"
    else
        printOK "Swap Memory: Off"
    fi

    if [ ! -z "$MEM_STATUS" ]; then
        printWarning "$MEM_STATUS"
    else
        printOK "Memory OK: $FREE_MEM GB Available"
    fi

    if [ ! -z "$DISK_ERROR" ]; then
        printError "$DISK_ERROR"
    else
        if [ ! -z "$DISK_WARNING" ]; then
            printWarning "$DISK_WARNING"
        else
            FREE_GB=$(( FREE_SPACE / 1048576))
            printOK "Disk OK: $FREE_SPACE Available"
        fi
    fi

    if [ ! -z "$OVERCOMMIT_MEM_STATUS" ]; then
        printError "$OVERCOMMIT_MEM_STATUS"
    else
        printOK "vm.overcommit_memory OK: $OVERCOMMIT_MEM"
    fi

    if [ ! -z "$KPANIC_STATUS" ]; then
        printError "$KPANIC_STATUS"
    else
        printOK "kernel.panic OK: $KPANIC"
    fi

    if [ ! -z "$KPANIC_ON_OOPS_STATUS" ]; then
        printError "$KPANIC_ON_OOPS_STATUS"
    else
        printOK "kernel.panic_on_oops OK: $KPANIC_ON_OOPS"
    fi

    if [ ! -z "$DHCP_STATUS" ]; then
        printWarning "DHCP detected. If this is a managed route and IP will remain fixed, you may continue."
        printWarning "Kubernetes requires a fixed IP, so if the IP address of this server is likely to change,"
        printWarning " exit the installation and configure a static IP manually. See the REAMDE for details."
    else
        printOK "Assuming a fixed IP address, DHCP not detected"
    fi

    # If errors, print message & exit.
    if [ ! -z "$HAS_ERRORS" ]; then
        printError "Fix the above errors before continuing."
        exit 1
    fi
    # if warnings, print message and allow cancel
    if [ ! -z "$HAS_WARNINGS" ]; then
        printWarning "Some warnings were detected."
        echo "You probably should address these issues before continuing,"
        echo " but we can go ahead and try the installation anyway."
        echo "Press Y to continue anyway, otherwise this script will timeout & cancel in 20 seconds..."
        read -t 20 -p "Continue? (y/N)? " answer
        [ -z "$answer" ] && answer="No" # defualt to No, user must explicitly type something begining with y to continue
        echo ""
        if echo "$answer" | grep -iq "^y"; then
            printOK "OK, continuing with installation."
        else
            printError "Cancelling installation."
            exit 1
        fi
    fi

    # If we get this far, then checklist complete
    printOK "All brown M&Ms removed, let's do this... "

else
    # If not a terminal, then non interactive mode
    # Log details and set defaults
    echo "Non interactive mode."
    echo "Installing with default set up"
    echo
    # If errors - exit & report
    if [ ! -z "$HAS_ERRORS" ]; then
        echo "Installation cancelled, Fix the follow issues then try again try again."
        echo $SUDO_STATUS
        echo $OS_STATUS
        echo $SWAP_STATUS
        echo $DISK_ERROR
        exit 1
    fi
    # if warnings, print message and continue anyway
    if [ ! -z "$HAS_WARNINGS" ]; then
        echo "Warnings were detected, but installation may proceed."
        echo "If any issues are encountered, review the following warnings which may help resolve the issue:"
        echo $MEM_STATUS
        echo $DISK_WARNING
        echo $DHCP_STATUS
        echo
    fi
fi

mkdir -p logs/

if [ "Version: $KUBE_VERSION" != "$KUBE_EXISTING_VERSION" ]; then
########################################################## STAGE 2: INSTALL BASE REQUIREMENTS
if [ ! -z "$KUBE_INSTALLED" ]; then
    if [ ! -z "$IS_TERMINAL" ]; then
        # reset existing cluster running old k8s version
        echo "The existing cluster needs to be reset as it is running an old version of Kubernetes"
        echo "Press y to reset and continue the installation, otherwise this script will timeout & cancel in 20 seconds..."
        read -t 20 -p "Continue? (y/n)? " answer
        [ -z "$answer" ] && answer="n" # defualt to No, user must explicitly type something begining with y to continue
        echo ""
        if echo "$answer" | grep -iq "^y"; then
            printOK "OK, continuing with installation."
        else
            printError "Cancelling installation. You can run reset-cluster.sh before re-running this script to reset the existing cluster manually."
            exit 1
        fi
    fi
    bash ./reset-cluster.sh
fi

# Double check prerequisites:
apt update
apt -y upgrade
apt -y install build-essential git curl net-tools
# Install extras needed by docker & kube:
apt -y install apt-transport-https ca-certificates gnupg-agent software-properties-common
# Install filesystem/mount helpers:
apt -y install nfs-common cifs-utils

# 1. Docker install & enable
curl -fsSL https://download.docker.com/linux/$ID/gpg | apt-key add -
add-apt-repository "deb [arch=amd64] https://download.docker.com/linux/$ID $VERSION_CODENAME stable"
apt-get update
apt-get -y install docker-ce=$DOCKER_VERSION containerd.io
apt-mark hold docker-ce containerd.io

# Set up service to disable swap on startup
cp davra_setup.sh /usr/bin/davra_setup.sh
chmod +x /usr/bin/davra_setup.sh
cp davra_setup.service /etc/systemd/system/davra_setup.service
chmod 644 /etc/systemd/system/davra_setup.service
systemctl enable davra_setup


systemctl daemon-reload
systemctl start docker
systemctl enable --now docker

usermod -aG docker $SUDO_USER

# 2. Install Kubernetes
# As a requirement for your Linux Node's iptables to correctly see bridged traffic,
#  you should ensure net.bridge.bridge-nf-call-iptables is set to 1 in your sysctl config, e.g.
cat <<EOF | tee /etc/sysctl.d/k8s.conf
net.bridge.bridge-nf-call-ip6tables = 1
net.bridge.bridge-nf-call-iptables = 1
EOF
sysctl --system

curl -s https://packages.cloud.google.com/apt/doc/apt-key.gpg | apt-key add -
# cat ./apt-key.gpg | apt-key add -

# kube* 1.19.9 packages are no longer available via apt
# Download an archived copy of kubeadm, kubelet and kubectl from
# an s3 bucket
curl -O https://s3.amazonaws.com/downloads.davra.com/deb/k8s1.19.9.tar.gz
tar -zxf k8s1.19.9.tar.gz
apt install ebtables conntrack
dpkg -i k8s1.19.9/cri-tools_1.26.0-00_amd64.deb
dpkg -i k8s1.19.9/kubernetes-cni_1.2.0-00_amd64.deb
dpkg -i k8s1.19.9/kubelet_1.19.9-00_amd64.deb
dpkg -i k8s1.19.9/kubectl_1.19.9-00_amd64.deb
dpkg -i k8s1.19.9/kubeadm_1.19.9-00_amd64.deb

#
# Extras:
#
# Install JQ, # will need this later:
# https://stedolan.github.io/jq/download/
apt-get install -y jq


# Initialise Kube
# flannel needs this --pod-network-cidr 10.244.0.0/16 apparently
# --service-cidr 10.3.0.0/24 needed for microservice architecture
if [ ! -z "$IS_TERMINAL" ]; then
    printOK "#############################################################"
    printOK " This next bit will take a few quiet minutes "
    printOK "  go grab a coffee and wait for the done message "
    printOK " Outputs are piped to log dir to keep a proper report & 'join' cammand if/when needed "
    printOK "#############################################################"
    kubeadm init --pod-network-cidr $POD_NETWORK_CIDR --service-cidr $SERVICE_NETWORK_CIDR >logs/kubeadmin.init.log  2>&1

    KUBEADM_ERRORS=$(grep 'ERROR' localhost/logs/kubeadmin.init.log)
    if [ ! -z "$KUBEADM_ERRORS" ]; then
        printError "Errors found in KubeAdm installation:"
        printError "$KUBEADM_ERRORS"
        printError "See localhost/logs/kubeadmin.init.log for full details to address the error and trya again."
        exit 1
    fi

else
    kubeadm init --pod-network-cidr $POD_NETWORK_CIDR --service-cidr $SERVICE_NETWORK_CIDR
fi

crontab -l > cron.tmp
if ! grep -q "kubeadm alpha certs renew all" cron.tmp; then
    KUBEADM_CERT_RENEW_CRON=${KUBEADM_CERT_RENEW_CRON:-"0 1 * * *"}
    echo "$KUBEADM_CERT_RENEW_CRON kubeadm alpha certs renew all" >> cron.tmp
    crontab cron.tmp
    rm cron.tmp
fi

until [ -f /etc/kubernetes/admin.conf ]; do
    echo "Waiting for conf file..."
    sleep 5
done
printOK "...conf file found, setting configs for user & root: "

USER_KUBE_ROOT=/home/$SUDO_USER/.kube
mkdir -p $USER_KUBE_ROOT
cp /etc/kubernetes/admin.conf $USER_KUBE_ROOT/config
chown $SUDO_USER:$SUDO_USER $USER_KUBE_ROOT/config

# Set kube conf for root/sudo:
mkdir -p /root/.kube
cp /etc/kubernetes/admin.conf /root/.kube/config

# and because it'll still be a prick about it....
export KUBECONFIG=/etc/kubernetes/admin.conf
cp /etc/kubernetes/admin.conf ./kubeconfig

kubectl taint nodes --all node-role.kubernetes.io/master- >/dev/null  2>&1

printOK "Now waiting for kubernetes to become available"
waitForKubernetes
echo
if [ $? -ne 0 ]; then
    printError "Installation of kubernetes failed exiting installation"
    exit 1
else
    printOK "Kubernetes install complete"
fi

printOK "Installing Flannel: "
sed s/POD_NETWORK/$POD_NETWORK/g kube-flannel.yaml.tpl > kube-flannel.yaml
sed -i s/POD_NW_MASK/${POD_NETWORK_MASK}/g kube-flannel.yaml
kubectl apply -f ./kube-flannel.yaml

printOK "Label the nodes for the daemonset to run..."
NODE_NAME=$(kubectl get node | grep master | awk '{print $1}')
kubectl label node $NODE_NAME kubernetes.io/os=linux

printOK "scale the coredns component to 1 replica"
kubectl scale deployment coredns -n kube-system --replicas=1

printOK "Disable leader election in kube-controller-manager and kube-schedule-manager"
sed -i s/leader\-elect=true/leader\-elect=false/g /etc/kubernetes/manifests/kube-controller-manager.yaml
sed -i s/leader\-elect=true/leader\-elect=false/g /etc/kubernetes/manifests/kube-scheduler.yaml

printOK "set min TLS version in kube-controller-manager and kube-schedule-manager"
sed -ie '/- kube-controller-manager/a \    - --tls-min-version=VersionTLS12' /etc/kubernetes/manifests/kube-controller-manager.yaml
sed -ie '/- kube-scheduler/a \    - --tls-min-version=VersionTLS12' /etc/kubernetes/manifests/kube-scheduler.yaml

sed -ie '/- etcd/a \    - --peer-auto-tls=false' /etc/kubernetes/manifests/etcd.yaml
sed -ie '/- etcd/a \    - --auto-tls=false' /etc/kubernetes/manifests/etcd.yaml

echo "readOnlyPort: 0" >> /var/lib/kubelet/config.yaml
echo "protectKernelDefaults: true" >> /var/lib/kubelet/config.yaml

printOK "set DynamicKubeletConfig to false"
echo "featureGates:" >> /var/lib/kubelet/config.yaml
echo "  DynamicKubeletConfig: false" >> /var/lib/kubelet/config.yaml
sed -ie '/- kube-scheduler/a \    - --feature-gates=DynamicKubeletConfig=false' /etc/kubernetes/manifests/kube-scheduler.yaml
sed -ie '/- kube-controller-manager/a \    - --feature-gates=DynamicKubeletConfig=false' /etc/kubernetes/manifests/kube-controller-manager.yaml
sed -ie '/- etcd/a \    - --feature-gates=DynamicKubeletConfig=false' /etc/kubernetes/manifests/etcd.yaml

printOK "audit policy file"
mkdir /etc/kubernetes/audit

cp audit-policy.yaml /etc/kubernetes/audit/audit-policy.yaml

kubectl create -f pod-security-policy.yaml

sed -ie '/- kube-controller-manager/a \    - --profiling=false' /etc/kubernetes/manifests/kube-controller-manager.yaml

printOK "Fixup kube-apiserver"
INTERNALIP=$(kubectl get node -o wide | tail -1 | awk '{print $6}')
cp /etc/kubernetes/manifests/kube-apiserver.yaml /home/admin/apiserver.yaml.backup

SECRET_NAME=$(kubectl get secret -n kube-system | grep "default-token" | awk '{print $1}')

while [ -z "$SECRET_NAME" ]; do
    sleep 30
    SECRET_NAME=$(kubectl get secret -n kube-system | grep "default-token" | awk '{print $1}')
done

TOKEN=$(kubectl get secret -n kube-system $SECRET_NAME -o jsonpath='{.data.token}' | base64 --decode)

sed s/INTERNALIP/$INTERNALIP/g apiserver.yaml.tpl | sed s/TOKEN/$TOKEN/g > /etc/kubernetes/manifests/kube-apiserver.yaml

printOK "Wait for api-server to come back up"
waitForKubernetes

echo
printOK "Set default label"
kubectl label namespace default tenantId=ruban
export CREATE_RUBAN_TENANT="true"

# TEST WITHOUT REBOOT
# drain, restart servuces & uncordon instead of rebooting..
TIMEOUT_IN_MINUTES=30
TIMEOUT_IN_SECONDS=$(expr $TIMEOUT_IN_MINUTES \* 60)
TIMEOUT_SLEEP_TIME=3
TIMEOUT_IN_LOOPS=$(expr $TIMEOUT_IN_SECONDS / $TIMEOUT_SLEEP_TIME)
NOT_TIMEOUT=$(expr $TIMEOUT_IN_LOOPS + 100)
PASS_THRESHOLD=30

function waitForK8SAPI() {
    echo "Waiting for kubernetes API..."
    cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]; do
        kubectl get ns >/dev/null 2>&1
        if [ $? -eq 0 ]; then
            cnt=$(expr $NOT_TIMEOUT - 1)
        fi
        sleep $TIMEOUT_SLEEP_TIME
        printf "+"
        cnt=$(expr $cnt + 1)
    done
    echo

    if [ $cnt -ne $NOT_TIMEOUT ]; then
        printError "TIMEOUT! k8s api not responding, Kubernetes is not working properly, examine journalctl -u kubelet for more info"
        exit 1
    fi
}

NODE=$(kubectl get node | tail -n+2 | awk '{print $1}')
kubectl drain $NODE --force --delete-local-data --ignore-daemonsets
systemctl stop kubelet
systemctl restart docker
systemctl start kubelet
waitForK8SAPI
kubectl uncordon $NODE
waitForKubernetes

#create the kubeconfig file
GET_EXTERNAL_IP_FROM_META=${GET_EXTERNAL_IP_FROM_META:-false}
KUBE_MASTER_HOST=$MASTER_HOST
if [ $GET_EXTERNAL_IP_FROM_META = "true" ] ; then
    KUBE_MASTER_HOST=`curl http://169.254.169.254/latest/meta-data/public-ipv4`
    echo "########### META DATA ############"
    echo "# public-ipv4: $KUBE_MASTER_HOST"
    INSTANCE_TYPE=`curl http://169.254.169.254/latest/meta-data/instance-type`
    echo "# instance-type: $INSTANCE_TYPE"
    echo "##################################"
fi

# Make a copy of this good kubeconfig which has the external ip because it may get overwritten later.
echo "copying kubeconfig to kubeconfig-using-external-ip"
# ls -l kubeconfig kubeconfig-using-external-ip
cp /home/admin/.kube/config kubeconfig-using-external-ip
kubectl config set-cluster kubernetes --server=https://$KUBE_MASTER_HOST:6443 --insecure-skip-tls-verify=true --kubeconfig=kubeconfig-using-external-ip


#
# silently remove any taints, and go to phase 2:
#
kubectl taint nodes --all node-role.kubernetes.io/master- >/dev/null  2>&1

export PATH=$(pwd):$PATH
export FLUENT_CONFIGMAP_TPL=fluent-bit-onprem-json-file-configmap.yaml.tpl

#Ensure loopback is in the hosts file
if [ ! -f "/etc/hosts" ]; then
    echo "127.0.0.1 localhost" >/etc/hosts
else
    if ! grep -q localhost /etc/hosts; then
        echo "127.0.0.1 localhost" >>/etc/hosts
    fi
fi

#Configure rate limiting on journald
cp /etc/systemd/journald.conf /etc/systemd/journald.conf.$(date +%s)
sed -i s/#RateLimitIntervalSec=30s/RateLimitIntervalSec=0/g /etc/systemd/journald.conf
systemctl restart systemd-journald

fi

########################################################## STAGE 3: DEPLOY PLATFORM
#
# do deploy
if [ ! -z "$IS_TERMINAL" ]; then
    printOK "Main Davra Deploy, outputs logged to logs/davra.deploy "
    /bin/bash ./deploy.sh >logs/davra.deploy.log 2>&1
else
    /bin/bash ./deploy.sh
fi
if [ $? -ne 0 ] ; then
    printError "Failed to Deploy"
    exit 1
fi

cnt=0

TIMEOUT_IN_MINUTES=30
TIMEOUT_IN_SECONDS=$(expr $TIMEOUT_IN_MINUTES \* 60)
TIMEOUT_SLEEP_TIME=3
TIMEOUT_IN_LOOPS=$(expr $TIMEOUT_IN_SECONDS / $TIMEOUT_SLEEP_TIME)
NOT_TIMEOUT=$(expr $TIMEOUT_IN_LOOPS + 100)
PASS_THRESHOLD=10

#Deploy kubelet memory usage monitor
cp ../monitoring/kubelet-monitor/kubelet-check.sh /root
cp ../monitoring/kubelet-monitor/kubelet-monitor.service /etc/systemd/system/
cp ../monitoring/kubelet-monitor/kubelet-monitor.timer /etc/systemd/system/
systemctl enable kubelet-monitor.timer
systemctl start kubelet-monitor.timer

function waitForKubernetes() {
    printOK "Waiting for kubernetes..."
    columns=NAMESPACE:.metadata.namespace,NAME:.metadata.name,REPLICAS:.status.replicas,AVAILABLE:.status.availableReplicas,LABELS:.metadata.labels
    cnt=0
    pass_cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]; do
        #kubectl get ns > /dev/null 2>&1
        kubectl get deployment --all-namespaces -o custom-columns=$columns | tail -n+2 | grep -v 'connecthing.io/microservice' | awk '{print $3-$4}' | grep -v 0 >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            pass_cnt=$(expr $pass_cnt + 1)
            if [ $pass_cnt -ge $PASS_THRESHOLD ]; then
                cnt=$(expr $NOT_TIMEOUT - 1)
            fi
        else
            #echo "Resetting pass counter to zero"
            pass_cnt=0
        fi
        sleep $TIMEOUT_SLEEP_TIME
        printf "-"
        cnt=$(expr $cnt + 1)
    done
    echo

    if [ $cnt -ne $NOT_TIMEOUT ]; then
        printError "TIMEOUT! Kubernetes is not working properly, examine journalctl -u kubelet for more info"
        exit 1
    fi

    printOK "$pass_cnt successive passes for deployment checks, all is well"
}

########################################################## STAGE 4: INSTALL MIRCO SERVICES
#
printOK "########################################################################################"
printOK "Deploying microservices"
printOK "This may take a long time: 10-30 mins or longer depending on your internet connection."
printWarning "Do not stop the installation."
printOK "########################################################################################"

export PATH=$PATH:$(pwd)

if [ ! -z "$IS_TERMINAL" ]; then
    printOK "Install Microservice Infrastructure, outputs logged to logs/microservice.infrastructure "
    bash ../infrastructure/install-microservice-infrastructure.sh >logs/microservice.infrastructure.log 2>&1
else
    bash ../infrastructure/install-microservice-infrastructure.sh
fi
if [ $? -ne 0 ] ; then
    printError "Failed to Install Microservice Infrastructure"
    exit 1
fi

printOK "########################################################################################"
printOK "Waiting for deployments to come up."
printOK "This may take a long time: 10-30 mins or longer depending on your internet connection."
printWarning "Do not stop the installation."
printOK "########################################################################################"
echo

waitForKubernetes

##########################################################
# CHECK RBAC
##########################################################
RBAC_ENABLED=$(cat /etc/kubernetes/manifests/kube-apiserver.yaml | grep "Node,RBAC")
if [ -z "$RBAC_ENABLED" ]; then
    echo "------------------------------------------------------------------------"
    echo "WARNING"
    echo "RBAC authorization mode is disabled, you will need to enable it manually"
    echo "You can do this by editing /etc/kubernetes/manifests/kube-apiserver.yaml"
    echo "and setting - --authorization-mode=Node,RBAC"
    echo "------------------------------------------------------------------------"
fi

printOK "####### Done! #######"
