#!/bin/bash

cd `dirname $0`

#kubernetes variables
export PRIMARY_IFACE=`route |grep default | head -1 | awk '{print $NF}'`
export MASTER_HOST=`ifconfig $PRIMARY_IFACE | grep inet | grep -v inet6 | awk '{print $2}'`
export ETCD_ENDPOINTS="http://127.0.0.1:2379"
export _ETCD_ENDPOINTS="http:\/\/127.0.0.1:2379"
export POD_NETWORK=${POD_NETWORK:-"10.2.0.0/16"}
export SERVICE_IP_NW=10.3.0.0
export SERVICE_IP_MASK=24
export SERVICE_IP_RANGE=$SERVICE_IP_NW/$SERVICE_IP_MASK
export K8S_SERVICE_IP=10.3.0.1
export DNS_SERVICE_IP=10.3.0.10
export K8S_VER=v1.12.9
export K8S_CORE_VER=$K8S_VER
echo "MASTER_HOST: $MASTER_HOST"
echo "ETCD_ENDPOINTS: $ETCD_ENDPOINTS"

TIMEOUT_IN_MINUTES=30
TIMEOUT_IN_SECONDS=`expr $TIMEOUT_IN_MINUTES \* 60`
TIMEOUT_SLEEP_TIME=3
TIMEOUT_IN_LOOPS=`expr $TIMEOUT_IN_SECONDS / $TIMEOUT_SLEEP_TIME`
NOT_TIMEOUT=`expr $TIMEOUT_IN_LOOPS + 100`

export PATH=`pwd`:$PATH

#kubectl
if [ ! -f kubectl ] ; then
	KUBECTL_URL="https://storage.googleapis.com/kubernetes-release/release/$K8S_VER/bin/linux/amd64/kubectl"
	curl -f -O $KUBECTL_URL
	if [ $? -ne 0 ] ; then
		echo "Failed to download kubectl from $KUBECTL_URL"
		echo "Please download this file and place it at "`pwd`"and try the install again"
		exit 1
	fi
	chmod +x kubectl
fi

#loops waiting for k8s to come up
function waitForKubernetes {
	echo "Waiting for kubernetes..."
	cnt=0
	while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
	do
		kubectl get ns > /dev/null 2>&1
	    if [ $? -eq 0 ] ; then
	        cnt=`expr $NOT_TIMEOUT - 1`
	    fi
	    sleep $TIMEOUT_SLEEP_TIME
	    printf "-"
	    cnt=`expr $cnt + 1`
	done
	echo

	if [ $cnt -ne $NOT_TIMEOUT ] ; then
		echo "TIMEOUT! Kubernetes is not working properly, examine journalctl -u kubelet for more info"
		exit 1
	fi
}

function startDNS {
	#kube-dns
	kubectl get pod --namespace=kube-system | grep kube-dns > /dev/null 2>&1
	if [ $? -ne 0 ] ; then
		echo "Deploying kubernetes dns"
		sed s/DNS_SERVICE_IP/$DNS_SERVICE_IP/g dns-addon.yaml.tpl > dns-addon.yaml
		kubectl create -f dns-addon.yaml
	fi
}

function bounceDNS {
	echo "Bouncing kubernetes dns"

	#need a nicer place to put this
	export PATH=$PATH:`pwd`
	kubectl get secret | tail -n+2 | awk '{system("kubectl delete secret "$1)}' 
	kubectl get deployment | tail -n+2 | awk '{system("kubectl scale deployment "$1" --replicas=0")}'

	#Bounce the kube dns
	kubectl get secret --namespace=kube-system | grep default | awk '{system("kubectl delete secret "$1" --namespace=kube-system")}'
	kubectl scale rc kube-dns-v20 --namespace=kube-system --replicas=0

	#Wait for kube dns to stop
	DNS_UP=0
	echo "Waiting for kubernetes dns to stop..."
	while [ $DNS_UP -eq 0 ]
	do
		kubectl get pod --namespace=kube-system | grep kube-dns | grep -i Running > /dev/null 2>&1 
		DNS_UP=$?
		printf ">"
		sleep 1
	done

	kubectl scale rc kube-dns-v20 --namespace=kube-system --replicas=1

	#Wait for kube dns to come up
	echo "Waiting for kubernetes dns to start..."
	while [ $DNS_UP -ne 0 ]
	do
		kubectl get pod --namespace=kube-system | grep kube-dns | grep -i Running > /dev/null 2>&1 
		DNS_UP=$?
		printf ">"
		sleep 1
	done
}

#configures and runs k8s master components
function doK8sMasterComponents {

	echo "Configuring kubernetes master components"
	mkdir -p /etc/kubernetes/manifests

	uuu=0
	kubectl get ns > /dev/null
	if [ $? -eq 0 ] ; then 
		uuu=1
		echo "Draining node for maintenance"
		kubectl get node | tail -n+2 | awk '{system("kubectl drain --force --delete-local-data "$1)}'
	fi

	#stop kubelet and docker
	echo "Stopping k8s and docker"
	systemctl stop kubelet > /dev/null 2>&1
	systemctl stop docker 

	#kube-apiserver
	sed s/ETCD_ENDPOINTS/$_ETCD_ENDPOINTS/g kube-apiserver.yaml.tpl | sed s/ADVERTISE_IP/$MASTER_HOST/g | sed s/K8S_VER/$K8S_CORE_VER/g | sed s/SERVICE_IP_NW/$SERVICE_IP_NW/g  | sed s/SERVICE_IP_MASK/$SERVICE_IP_MASK/g > /etc/kubernetes/manifests/kube-apiserver.yaml

	#kube-proxy
	sed s/K8S_VER/$K8S_CORE_VER/g kube-proxy.yaml.tpl > /etc/kubernetes/manifests/kube-proxy.yaml

	#kube-controller-manager
	sed s/K8S_VER/$K8S_CORE_VER/g kube-controller-manager.yaml.tpl > /etc/kubernetes/manifests/kube-controller-manager.yaml

	#kube-scheduler
	sed s/K8S_VER/$K8S_CORE_VER/g kube-scheduler.yaml.tpl > /etc/kubernetes/manifests/kube-scheduler.yaml

	#Kubelet
	sed s/K8S_VER/$K8S_CORE_VER/g kubelet.service.tpl | sed s/ADVERTISE_IP/$MASTER_HOST/g | sed s/DNS_SERVICE_IP/$DNS_SERVICE_IP/g > /etc/systemd/system/kubelet.service
	cp master-kubeconfig.yaml /etc/kubernetes/manifests

	systemctl daemon-reload
	systemctl enable kubelet

	#start kubelet and docker
	systemctl start docker
	systemctl start kubelet

	if [ $uuu -eq 1 ] ; then
		waitForKubernetes
		echo "Kubernetes configured - Uncordoning node"
		kubectl get node | tail -n+2 | awk '{system("kubectl uncordon "$1)}'

		# HACK: In some RuBAN upgrade scenarios we have seen etcd get totally out of sync with what kubectl reports
		# That is, etcdctl will list all pods but kubectl only knows about kube-system's gang of four (kube-apiserver, kube-controller-manager, kube-proxy & kube-scheduler)
		# We definitely need DNS (!) so we ensure it is started here - and because the subsequent install will (re)create all pods that kubectl should already know about, 
		# we get away with just this action here....
		#  *** BUT LET'S BE ABSOLUTELY CLEAR, this is a total cludge and the real problem is that etcd is failing  (probably due to our single node deployment which breaks all the recommendation for Kubernetes) ***
		startDNS

	fi
}

function refreshCerts {
	echo "Refreshing Kubernetes certs..."
    #regenerate the apiserver cert
    /bin/bash tls_gen.sh
    
    #restart the apiserver
    kubectl get pod -n kube-system | grep kube-apiserver | awk '{system("kubectl delete pod "$1" -n kube-system")}'
    sleep 10
    waitForKubernetes
    
    #delete the stale secrets
    kubectl get secret -n kube-system | grep default | grep kubernetes | awk '{system("kubectl delete secret "$1" -n kube-system")}'
    kubectl get secret -n default | grep default | grep kubernetes | awk '{system("kubectl delete secret "$1" -n default")}'
    
    #bounce the dns deployment
    kubectl scale rc kube-dns-v20 -n kube-system --replicas=0
    kubectl scale rc kube-dns-v20 -n kube-system --replicas=1
    kubectl scale rc kubernetes-dashboard-v1.4.1 -n kube-system --replicas=0
    kubectl scale rc kubernetes-dashboard-v1.4.1 -n kube-system --replicas=1
}

#If upgrading and there is an ip address change reinstall k8s else just return
UPGRADE=1
IP_CHANGED=1
K8S_OLD=1
CERTS_EXPIRED=1
if [ -f /etc/kubernetes/manifests/kube-apiserver.yaml ] ; then
	UPGRADE=0

	#Change in IP ?
	grep "$MASTER_HOST" /etc/kubernetes/manifests/kube-apiserver.yaml
	if [ $? -ne 0 ] ; then
		IP_CHANGED=0
	fi

	#k8s needs upgrade?
	grep "$K8S_CORE_VER" /etc/kubernetes/manifests/kube-apiserver.yaml
	if [ $? -ne 0 ] ; then
		K8S_OLD=0
	fi

	#k8s template needs refresh
	grep "hostPort: 443" /etc/kubernetes/manifests/kube-apiserver.yaml
	if [ $? -eq 0 ] ; then
		K8S_OLD=0
	fi
	
	grep "var-lib-cni" /etc/systemd/system/kubelet.service
	if [ $? -ne 0 ] ; then
	    K8S_OLD=0
	fi
	
	#certs out of date
	EXPIRE_DATE=`openssl x509 -in /etc/kubernetes/ssl/apiserver.pem -text | grep "Not After" | awk '{print $4" "$5" "$6" "$7" "$8}'`
	EXPIRE_DATE_EPOC=`date -d "$EXPIRE_DATE" +%s`
	NOW=`date +%s`
	if [ $NOW -ge $EXPIRE_DATE_EPOC ] ; then
	    CERTS_EXPIRED=0
	fi
fi

#remove k8s dashboard if present
kubectl get rc --namespace=kube-system kubernetes-dashboard-v1.4.1 > /dev/null 2>&1
if [ $? -eq 0 ] ; then
	kubectl delete rc kubernetes-dashboard-v1.4.1 -n kube-system
fi

#etcd:  Repair if broken or Migrate
/bin/bash repair_etcd.sh
REP_RES=$?
if [ $REP_RES -ne 0 -a $REP_RES -ne 99 ] ; then
	exit 1
fi
if [ $REP_RES -eq 99 ] ; then
	# Migrating etcd2 to etcd3
    waitForKubernetes
fi

function refreshCerts {
    
    #regenerate the apiserver cert
    /bin/bash tls_gen.sh
    
    systemctl stop kubelet
    systemctl restart docker
    systemctl start kubelet
    waitForKubernetes
    
    echo "Waiting for DNS"
    DNS_UP=1
	while [ $DNS_UP -ne 0 ]
	do
		kubectl get pod --namespace=kube-system | grep kube-dns | grep -i "3/3" > /dev/null 2>&1 
		DNS_UP=$?
		printf ">"
		sleep 1
	done
	echo
}

#nothing to do - exit
if [ $UPGRADE -eq 0 -a $K8S_OLD -eq 1 -a $IP_CHANGED -eq 1 -a $CERTS_EXPIRED -eq 1  ] ; then
    echo "Kubernetes OK"
	exit 0
fi

#certs expired
if [ $CERTS_EXPIRED -eq 0 ] ; then
    echo "Certs expired refreshing"
	refreshCerts
fi

#no ip change but k8s components need upgrading
if [ $K8S_OLD -eq 0 -a $IP_CHANGED -eq 1 ] ; then
	echo "Upgrade on same IP requires Kubernetes version update"
	doK8sMasterComponents
fi

if [ \( $CERTS_EXPIRED -eq 0 -o $K8S_OLD -eq 0 \) -a $IP_CHANGED -eq 1 ] ; then
    echo "Kubernetes OK"
    exit 0
fi

#ip changed clear out old cni0 interface and install as if fresh install
if [ $IP_CHANGED -eq 0 ] ; then
	echo "Upgrade on different IP requires clearing out of old cni0 interface"
	systemctl stop kubelet
	systemctl stop docker
	systemctl stop flanneld
	ifconfig cni0 down
	brctl delbr cni0
fi

#regenerate the apiserver cert
/bin/bash tls_gen.sh

#flanneld Config
/bin/bash configure_flannel.sh

#Configure docker
/bin/bash configure_docker.sh

#Download docker images
echo "Downloading container images"
ls -1 *.yaml* | awk '{system("grep -i image "$1)}' | awk '{print $NF}' | sed s/K8S_VER/$K8S_CORE_VER/g | sort | uniq | awk '{system("docker pull " $1)}'
if [ $? -ne 0 ] ; then
	echo "Failed to download images"
fi

#configure k8s manifest files and kubelet
doK8sMasterComponents

#create the kubeconfig file
GET_EXTERNAL_IP_FROM_META=${GET_EXTERNAL_IP_FROM_META:-false}
KUBE_MASTER_HOST=$MASTER_HOST
if [ $GET_EXTERNAL_IP_FROM_META = "true" ] ; then
    KUBE_MASTER_HOST=`curl http://169.254.169.254/latest/meta-data/public-ipv4`
    echo "########### META DATA ############"
    echo "# public-ipv4: $KUBE_MASTER_HOST"
    INSTANCE_TYPE=`curl http://169.254.169.254/latest/meta-data/instance-type`
    echo "# instance-type: $INSTANCE_TYPE"
    echo "##################################"
    
fi

CA_CERT=/etc/kubernetes/ssl/ca.pem
ADMIN_KEY=./admin-key.pem
ADMIN_CERT=./admin.pem
kubectl config set-cluster default-cluster --server=https://$KUBE_MASTER_HOST:8443 --embed-certs=true --certificate-authority=$CA_CERT --kubeconfig=kubeconfig
kubectl config set-credentials default-admin --certificate-authority=$CA_CERT --embed-certs=true --client-key=$ADMIN_KEY --client-certificate=$ADMIN_CERT --kubeconfig=kubeconfig
kubectl config set-context default-system --cluster=default-cluster --user=default-admin  --kubeconfig=kubeconfig
kubectl config use-context default-system --kubeconfig=kubeconfig
# Make a copy of this good kubeconfig which has the external ip because it may get overwritten later.
echo "copying kubeconfig to kubeconfig-using-external-ip"
ls -l kubeconfig kubeconfig-using-external-ip
cp kubeconfig kubeconfig-using-external-ip

#wait for api to come up
waitForKubernetes

#tag the default namespace as the 'ruban' tenant
kubectl describe ns default | grep Labels | grep tenantId
if [ $? -ne 0 ] ; then
	kubectl label namespace default tenantId=ruban
fi

#kube-dns
startDNS

#When ip changed we need to scale-down the dns service, clear the k8s default secret and scale back up
if [ $IP_CHANGED -eq 0 ] ; then
	bounceDNS
fi

#Kubernetes may already be installed but docker could be bounced so we need to wait for it to come up
printf "\nWaiting for kubernetes to become available\n"
waitForKubernetes
