'use strict';

const { executeCommand, sendAlert } = require('../monitoring.lib');

let cloud = 'aws/staging';
let roomId = 'Y2lzY29zcGFyazovL3VzL1JPT00vMGYyYTA5YzAtOTI2Yi0xMWU5LTkzZjMtNzU2OWE1YjlhZTVk';

process.argv.forEach((val) => {
  if(val.indexOf("--cloud=")>=0){
    cloud = val.substring(val.indexOf("=")+1);
  }
  else if(val.indexOf("--roomId=")>=0) {
    roomId = val.substring(val.indexOf("=")+1);
  }
});

const KUBECTL = `kubectl --kubeconfig=${__dirname}/../../${cloud}/kubeconfig`;
const FAILED_PODS_CMD = `${KUBECTL} get pod --all-namespaces --show-labels | tail -n+2 | grep -v jenkins-slave | grep -v node-exporter  | grep -v 'connecthing.io/microservice=true' | grep -v fluent-bit | grep -v workspace |  grep -v Running | grep -v Terminating | grep -v Completed | awk '{print $1"/"$2}'`;

/**
 * An AWS Lambda and API Gateway exist to proxy to a filtered Discover
 * view on Kibana, filtering logs for a given pod name. This was
 * done because Kibana URLs do not play well with markdown, so
 * alerting messages to Webex with URLs to Kibana resulted in unexpected
 * markdown parsing.
 *
 * The Lambda is titled "cloudMonitoringKibanaReportProxy" and lives at https://eu-west-1.console.aws.amazon.com/lambda/home?region=eu-west-1#/functions/cloudMonitoringKibanaReportProxy?newFunction=true&tab=configuration
 *
 * The API Gateway is titled "cloudMonitoringKibanaReportProxyAPI" and lives at https://eu-west-1.console.aws.amazon.com/apigateway/home?region=eu-west-1#/apis/le3pxscvsf/resources/d08mm8oxre
 *
 */
const AWS_LAMBDA_CLOUD_MONITORING_KIBANA_REPORT_PROXY_URL = 'https://le3pxscvsf.execute-api.eu-west-1.amazonaws.com/v1/pods';

/**
 * @return {[String]} List of failed pod identifiers in the format NAMESPACE/NAME
 */
async function getFailedPods() {
  let stdout = await executeCommand(FAILED_PODS_CMD);
  if (!stdout) return [];

  // "pod1\npod2\npod3\n" -> ["pod1", "pod2", "pod3"]
  // "" -> []
  return stdout.split('\n').filter(item => item !== '');
}

function parsePodKibanaLogsUrl(podName) {
  return `${AWS_LAMBDA_CLOUD_MONITORING_KIBANA_REPORT_PROXY_URL}/${podName}`;
}

/**
 * @param {*} pod - pod identifier in the format NAMESPACE/NAME
 * @return {String} markdown - Markdown containing pod identifer and a URL
 * to view the pod details in Kibana. Markdown is in the following format:
 * 
 *  default/davra-api-7fcf8db498-zc7t2
 *  - View Kibana Logs
 */
function parsePodMarkdown(pod) {
  const [namespace, podName] = pod.split('/');
  const podKibanaLogsUrl = parsePodKibanaLogsUrl(podName);
  return `${pod}<br/> - [View Kibana Logs](${podKibanaLogsUrl})`;
}

/**
 * @param {[String]} failedPods - list of pod identifiers
 * @return {String} markdown - markdown of an alert message for
 * failed pods in the format:
 * 
 * Wed Sep 30 2020 10:43:27 GMT+0000 (UTC) - failing deployments found on cloud aws/staging
 *
 * default/davra-api-7fcf8db498-zc7t2
 * - View Kibana Logs
 * 
 * default/kafka-0-8fcf8db496-ac7t3
 * - View Kibana Logs
 */
function parseAlertMessageMarkdown(failedPods) {
  const failedPodsMarkdown = failedPods.map(parsePodMarkdown).join('<br/><br/>');
  const timestamp = new Date();
  return `${timestamp} - failing deployments found on cloud ${cloud}<br/><br/>${failedPodsMarkdown}`;
}

async function run() {
  try {
    const failedPods = await getFailedPods();

    if (!failedPods.length) {
      console.log(`No failed pods detected on ${cloud}, exiting...`);
      process.exit(0);
    }
  
    console.log(`Failed pods detected on ${cloud}: ${failedPods.join(', ')}`)

    const markdown = parseAlertMessageMarkdown(failedPods);

    console.log(`Sending alert message: ${markdown}`);

    await sendAlert({
      roomId,
      markdown
    });

    console.log(`Alert message sent`);
  } catch(e) {
    console.error('Unexpected error occurred', e);
  }
}

run();