'use strict';

const axios = require('axios');
const https = require('https');
const UUID = require('node-uuid');
const _ = require("lodash");

const httpsAgent = new https.Agent({ rejectUnauthorized: false });

const METRIC_NAME = `ERS_Temperature-${parseInt(new Date().valueOf() * Math.random())}`;

const MOTE_TYPE_DECODER = `
var payload = new Buffer(context.payload, "base64");

var metrics = [];

//Example Payload:
//01 00 EF 02 30 04 01 9F 05 00 07 0D D4

var temp = (payload.readUInt16BE(1)*0.1);
var humid = payload.readUInt8(4);
var light = payload.readUInt16BE(6);
var motion = payload.readUInt8(9);
var batt = payload.readUInt16BE(11);

metrics.push({name: "` + METRIC_NAME + `", value: temp});
metrics.push({name: "ERS_humidity", value: humid});
metrics.push({name: "ERS_light", value: light});
metrics.push({name: "ERS_motion", value: motion});
metrics.push({name: "ERS_battery", value: batt});

context.value = {
		metrics: metrics,
		gps: {
						latitude: null,
						longitude: null
				}
};`;

const MOTE_TYPE_PAYLOAD = {
	name: "test-mote-type-" + parseInt(new Date().valueOf() * Math.random()),
	savePayload: true,
	decoder: {
		type: "raw",
		body: MOTE_TYPE_DECODER
	}
};

const DEVICE_PAYLOAD = {
	name: "lora-datum-test-" + (new Date().valueOf() * Math.random()),
	serialNumber: UUID.v4(),
	customTags: {
		"moteType_id": null, // we will set this once we have added the Mote Type
		"network_policy": "Standard"
	},
	labels: {
		devicelabelkey: `devicelabelvalue_${parseInt(Math.random() * new Date().valueOf())}`
	}
};

const TIMESTAMP = new Date();

const DATUM_PAYLOAD = {
	timestamp: TIMESTAMP.toISOString(),
	dataFrame: "AQDTAjUEAAIFAAcNSw==",
	fcnt: 255522,
	port: 5,
	rssi: -71,
	snr: 15,
	sf_used: 7,
	gtw_info: [
		{
			gtw_id: "0000000008050254",
			rssi: -102,
			snr: 8.8,
			gps_tmst: 1507849327620,
			freq: 868.1
		},
		{
			gtw_id: "005f86fffe5c28a0",
			rssi: -71,
			snr: 15,
			gps_tmst: 0,
			freq: 868.1
		}
	],
	id: 1507852928031,
	decrypted: true
};

function sleep(ms) {
  return new Promise(resolve => setTimeout(resolve, ms));
}

async function createMoteType(payload, options = {}) {
	const { baseUrl, username, password } = options;

	const response = await axios.post(`${baseUrl}/moteManager/api/v1/moteTypes`, payload, {
		auth: {
			username,
			password
		},
		httpsAgent
	});

	if (response.status !== 200) {
		throw 'Failed to create Mote Type';
	}

	return response.data;
}

async function deleteMoteType(id, options = {}) {
	const { baseUrl, username, password } = options;

	const response = await axios.delete(`${baseUrl}/moteManager/api/v1/moteTypes/${id}`, {
		auth: {
			username,
			password
		},
		httpsAgent
	});

	if (response.status !== 200) {
		throw `Failed to delete Mote Type: ${id}`;
	}

	return response.data;
}

async function createDevice(payload, options = {}) {
	const { baseUrl, username, password } = options;

	const response = await axios.post(`${baseUrl}/api/v1/devices`, payload, {
		auth: {
			username,
			password
		},
		httpsAgent
	});

	if (response.status !== 200) {
		throw 'Failed to create Device';
	}

	return response.data[0];
}

async function deleteDevice(uuid, options = {}) {
	const { baseUrl, username, password } = options;

	const response = await axios.delete(`${baseUrl}/api/v1/devices/${uuid}`, {
		auth: {
			username,
			password
		},
		httpsAgent
	});

	if (response.status !== 200) {
		throw 'Failed to delete Device';
	}

	return response.data;
}

function isUploadDatumException(uploadDatumResponseData = {}) {
	const { message = '', status = '' } = uploadDatumResponseData;
	if (message.includes('No device')) return true;
	if (message.includes('Error processing')) return true;
	return status !== 'ok';
}

async function uploadDatum(payload, options = {}) {
	const { baseUrl, username, password } = options;

	const response = await axios.post(`${baseUrl}/payloadManager/api/v1/process/payload/1`, payload, {
		auth: {
			username,
			password
		},
		httpsAgent
	});

	const data = response.data || {
		status: 'error',
		message: 'Unexpected error occurred, no response data received from uploading datum'
	};

	if (isUploadDatumException(data)) throw data.message;

	return data;
}

async function downloadDatum(query = {}, options = {}) {
	const {	deviceUUID, timestamp, metricName } = query;
	const { baseUrl, username, password } = options;

	const payload = {
		start_absolute: timestamp.valueOf() - 60000,
		end_absolute: timestamp.valueOf() + 60000,
		metrics: [{
			tags: {
				UUID: [deviceUUID]
			},
			name: metricName
		}]
	};

	const response = await axios.post(`${baseUrl}/api/v1/timeseriesData`, payload, {
		auth: {
			username,
			password
		},
		httpsAgent
	});

	const { data, status } = response;

	if (status !== 200) {
		throw `Status code: ${status} :: ${data}`;
	}

	if (data.queries[0].sample_size !== 1) {
		throw `Cannot find value for metric: ${metricName}`;
	}

	return data.queries[0].results[0];
}

/**
 * Sets up state that is needed for the Lora check to be performed. A Mote Type
 * is created, followed by a Device containing the ID of the previously created
 * Mote Type. 
 * 
 * @param {{ baseUrl: String, username: String, password: String }} options 
 * @return {{ moteType: MoteType, device: Device }} state for the Lora check
 */
async function setUp(options = {}) {
	console.log('Setting up...');

	console.log('Creating Mote Type...');
	const moteType = await createMoteType(MOTE_TYPE_PAYLOAD, options);
	console.log('Mote Type created:', JSON.stringify(moteType), '\n');

	console.log('Giving the Mote Type some time to persist...');
	await sleep(5000);

	console.log('Creating Device...');
	const devicePayload = _.cloneDeep(DEVICE_PAYLOAD);
	devicePayload.customTags.moteType_id = moteType._id;
	const device = await createDevice(devicePayload, options);
	console.log('Device created:', JSON.stringify(device), '\n');

	console.log('Giving the data collector some time to register the Device...');
	await sleep(10000);

	return { moteType, device };
}

/**
 * Uploads a datum for the device in state, gives it some time to persist,
 * then downloads the datum. If no exception is experienced during upload
 * and download, then check was a success, otherwise the exception
 * is propagated up the stack.
 * 
 * @param {{ device: Device }} state
 * @param {{ baseUrl: String, username: String, password: String }} options 
 */
async function runCheck(state = {}, options = {}) {
	console.log('Running check...');

	const { device } = state;

	const datumPayload = { ...DATUM_PAYLOAD, deveui: device.serialNumber };
	console.log('Uploading Datum...', JSON.stringify(datumPayload), '\n');
	const uploadDatumResponseData = await uploadDatum(datumPayload, options);
	console.log('Datum uploaded:', JSON.stringify(uploadDatumResponseData), '\n');

	console.log('Giving the data persistence service some time to save the Datum...')
	await sleep(10000);

	const query = {
		deviceUUID: device.UUID,
		timestamp: TIMESTAMP,
		metricName: METRIC_NAME
	};

	console.log('Downloading Datum...');
	const datum = await downloadDatum(query, options);
	console.log('Downloaded datum:', JSON.stringify(datum), '\n');
}

/**
 * cleans up state that is created for the Lora check to be performed. It
 * only cleans up whats needed, and does not throw an exception if any request
 * fails, as we want to clean up as much as possible.
 * 
 * @param {{ device: Device, motaType: MoteType }} state 
 * @param {{ baseUrl: String, username: String, password: String }} options 
 */
async function cleanUp(state = {}, options = {}) {
	console.log('Cleaning up...');

	const { device, moteType } = state;

	try {
		if (device) {
			try {
				console.log('Deleting Device...');
				await deleteDevice(device.UUID, options);
				console.log('Device deleted:', device.UUID, '\n');
			} catch (e) {
				console.error('Failed to delete Device during teardown:', device.UUID, e);
			}
		}

		if (moteType) {
			try {
				console.log('Deleting Mote Type....')
				await deleteMoteType(moteType._id, options);
				console.log('Mote Type deleted:', moteType._id, '\n');
			} catch (e) {
				console.error('Failed to delete Mote Type during teardown:', device.UUID, e);
			}
		}
	} catch (e) {
		console.error('Unexpected error during cleanup:', e);
	}
}

/**
 * Performs a Lora check, therefore verifying if iotdata flows through
 * the platform as expected.
 * 
 * State is set up by creating a Mote Type and Device.
 * 
 * The check is performed by creating a datum, giving it time to propagate,
 * and downloading the datum to ensure that the datum propagated as expected.
 * 
 * State is then cleaned up by deleting the Mote Type and Device.
 * 
 * A result is returned that indicates if the check was a success or not,
 * along with a human readable message and an error message if present.
 * If an exception occurs during the check, the check is deemed as a failure,
 * therefore the success field will be false, and the error that occurred will
 * be in the error field. Otherwise if no exceptions occur, the success field
 * will be true.
 * 
 * @param {{ baseUrl: String, username: String, password: String }} options 
 * @return {{ success: Boolean, message: String, error: String | Null }} result
 * details of the Lora check.
 */
async function loraCheck(options = {}) {
	const { baseUrl } = options;

	console.log(`Running loraCheck on ${baseUrl}`);

	let state, result;

	try {	
		state =  await setUp(options);
		await runCheck(state, options);

		result = {
			success: true,
			message: `No issues detected on ${baseUrl}`,
			error: null
		};
	} catch (e) {
		console.error(`Error on ${baseUrl}`, e);

		result = {
			success: false,
			message: `Issues detected on ${baseUrl}`,
			error: e
		};
	}
	
	await cleanUp(state, options);
	return result;
}

module.exports = loraCheck;