"use strict";

var http = require("http");
var fs = require("fs");
var util = require("util");

var yaml = require("node-yaml");


// Args, e.g.:
//    npm start "aaa-service"
// Also expects file pod.json to be sat in workspace directory (output from "kubectl get pod $POD -o json")
var yamlFile = "/yamls/"+process.argv[2]+"-deployment.yaml";
var revert = process.argv[3];  // Use with care!


// Get YAML file for pod
var yamlDeploy = yaml.readSync(yamlFile);
// Get current spec for pod, including any manually overridden resource settings
var jsonPod = JSON.parse(fs.readFileSync("/workspace/pod.json"));

// console.log("YAML returned:\n"+util.inspect(yamlDeploy, {depth: 8, colors: true }));
// console.log("POD returned:\n"+util.inspect(jsonPod, {depth: 8, colors: true }));

var namespace = jsonPod.metadata.namespace || "default";
var podName = jsonPod.metadata.name;


var ix = 0;
var numContainers = Object.keys(yamlDeploy.spec.template.spec.containers).length;
function processContainers() {
    if (ix >= numContainers) {
        console.log("Processed all "+numContainers+" containers in pod "+podName+", updating deployment YAML...");

        // Save (overwrite) deployment YAML
        yaml.writeSync(yamlFile, yamlDeploy);
        return;
    }

    var yamlContainer = yamlDeploy.spec.template.spec.containers[ix];
    console.log("Processing Pod: ["+namespace+" - "+podName+"]  Container: ["+yamlContainer.name+"]");

    // Highlight if no resources specified in installer YAML
    var installCPUReq, installCPULimit, installMemReq, installMemLimit;

    if (yamlContainer.resources) {
        if (yamlContainer.resources.limits) {
            installCPULimit = yamlContainer.resources.limits.cpu;
            installMemLimit = yamlContainer.resources.limits.memory;
        }

        if (yamlContainer.resources.requests) {
            installCPUReq = yamlContainer.resources.requests.cpu;
            installMemReq = yamlContainer.resources.requests.memory;
        }
    }

    var omissions = "";
    if (!yamlContainer.resources || !yamlContainer.resources.requests || !installCPUReq) {
        omissions += " no CPU Request";
    }
    if (!yamlContainer.resources || !yamlContainer.resources.limits || !installCPULimit) {
        omissions += ", no CPU Limit";
    }
    if (!yamlContainer.resources || !yamlContainer.resources.requests || !installMemReq) {
        omissions +=  ", no Memory Request";
    }
    if (!yamlContainer.resources || !yamlContainer.resources.limits || !installMemLimit) {
        omissions += ", no Memory Limit";
    }
    if (omissions !== "") {
        console.warn("Pod ["+namespace+" - "+podName+"] has"+omissions);
    }
    // console.log("installCPUReq="+installCPUReq+", installCPULimit="+installCPULimit+", installMemReq="+installMemReq+", installMemLimit="+installMemLimit);

  
    // Find container in pod (if not matched, will default to YAML values)
    var jsonContainer;
    if (jsonPod.spec) {
        jsonContainer = jsonPod.spec.containers.filter(function(item) {
            return item.name == yamlContainer.name;
        });
        jsonContainer = jsonContainer[0];
    }
    // console.log("Matched container:\n"+util.inspect(jsonContainer, {depth: 8, colors: true }));

    var localCPUReq, localCPULimit, localMemReq, localMemLimit;
    // Preserve any existing resource settings...
    if (jsonContainer && jsonContainer.resources) {
        if (jsonContainer.resources.limits) {
            localCPULimit = jsonContainer.resources.limits.cpu;
            localMemLimit = jsonContainer.resources.limits.memory;
        }

        if (jsonContainer.resources.requests) {
            localCPUReq = jsonContainer.resources.requests.cpu;
            localMemReq = jsonContainer.resources.requests.memory;
        }
    }

    // ...otherwise, use values specified in install YAMLs (if any)
    if (!jsonContainer || !jsonContainer.resources || jsonContainer.resources.requests || !localCPUReq || !installCPUReq) {
        localCPUReq =  installCPUReq;
    }
    if (!jsonContainer || !jsonContainer.resources || !jsonContainer.resources.limits || !localCPULimit || !installCPULimit) {
        localCPULimit = installCPULimit;
    }
    if (!jsonContainer || !jsonContainer.resources || !jsonContainer.resources.requests || !localMemReq || !installMemReq) {
        localMemReq = installMemReq;
    }
    if (!jsonContainer || !jsonContainer.resources || !jsonContainer.resources.limits || !localMemLimit || !installMemLimit) {
        localMemLimit = installMemLimit;
    }

    // console.log("localCPUReq="+localCPUReq+", localCPULimit="+localCPULimit+", localMemReq="+localMemReq+", localMemLimit="+localMemLimit);

    // Update deployment YAML with local values (may be the same)
    yamlContainer.resources =  { requests: { }, limits: { }};
    if (!revert) {
        if (localCPUReq) {
            yamlContainer.resources.requests.cpu = localCPUReq;
        }
        if (localCPULimit) {
            yamlContainer.resources.limits.cpu = localCPULimit;
        }
        if (localMemReq) {
            yamlContainer.resources.requests.memory = localMemReq;
        }
        if (localMemLimit) {
            yamlContainer.resources.limits.memory = localMemLimit;
        }
    } 
    else {
        yamlContainer.resources =  { };
    }
    console.log("New resources:\n"+util.inspect(yamlContainer.resources, {depth: 3, colors: true }));

    // Process next container in pod
    ix++;
    processContainers();
}


// Process first container in pod
processContainers();
