#!/bin/bash
######################################################################################
# Davra Troubleshooting Script
#  
# 1. Grab general system data, platform, cpu, mem, disk, etc
# 2. Get current running state of all pods
# 3. Get nodes & namespaces
# 4. for each pod & service, dump out a log file
# 5. scan and report any errors found
# 6. Create archive of complete output, to be sent for analysis
######################################################################################

# Default to using current/local kubeconfig, but if passed an argument it's probably
#  an alternate/remote kubeconfig to use:
CONFIG=$1
if [[ ! -z "$CONFIG" ]]; then
    if [[ -f "$CONFIG" ]]; then
        if grep -q "kind: Config" $CONFIG; then        
            echo "Using config: $CONFIG for:"
            cat $CONFIG | grep server
            export KUBECONFIG=$CONFIG
        else
            echo "Arg: $CONFIG is not a kubeconfig file. Skipping."
        fi
    else
        echo "Arg: $CONFIG is not a file. Skipping."
    fi
fi



# If debugging a current issue, assume it's happened in the past hour
LOG_DURATION="1h"
# ...but extend that if needed.
# You will likely want to limit the log dumps for long running servers though

HOME=$(pwd)
cd $(dirname $0)

NOW=$(date)

# set & clear output dir:
OUTPUT_DIR="debug"
rm -rf $OUTPUT_DIR
mkdir -p $OUTPUT_DIR 

# 1. general system info
cat >$OUTPUT_DIR/info.txt <<EOL
$NOW
------------------------------System Information----------------------------
Hostname:       $(hostname)
Manufacturer:   $(cat /sys/class/dmi/id/chassis_vendor)
Product Name:   $(cat /sys/class/dmi/id/product_name)
Version:        $(cat /sys/class/dmi/id/product_version)
Machine Type:   $(vserver=$(lscpu | grep Hypervisor | wc -l); if [ $vserver -gt 0 ]; then echo "VM"; else echo "Physical"; fi)
OS:             $(hostnamectl | grep "Operating System" | cut -d ' ' -f5-)
Kernel:         $(uname -a)
Architecture:   $(arch)
Processor Name: $(awk -F':' '/^model name/ {print $2}' /proc/cpuinfo | uniq | sed -e 's/^[ \t]*//')

System Main IPs:
$(hostname -I)

Active Users:
$(w)

Memory Usage:   $(free -h)

CPU Usage:      $(cat /proc/stat | awk '/cpu/{printf("%.2f%%\n"), ($2+$4)*100/($2+$4+$5)}' |  awk '{print $0}' | head -1)

Disk:
$(df . -Ph)
----------------------------------------------------------------------------

EOL

# 2. current running state:
kubectl get pods --all-namespaces -o wide >> $OUTPUT_DIR/info.txt
kubectl get svc --all-namespaces -o wide >> $OUTPUT_DIR/info.txt

# ...and display:
cat $OUTPUT_DIR/info.txt

# 3. get nodes:
for node in $(kubectl get nodes --output=jsonpath={.items..metadata.name}); do
    echo "Checking $node ..."
    kubectl describe node $node >  $OUTPUT_DIR/$node.log
    grep -i "ERROR" $OUTPUT_DIR/$node.log
done

# get namespaces, and for each...
for ns in $(kubectl get ns --output=jsonpath={.items..metadata.name}); do
    echo "Checking $ns ..."
    mkdir -p $OUTPUT_DIR/$ns/
    mkdir -p $OUTPUT_DIR/$ns/pods/
    mkdir -p $OUTPUT_DIR/$ns/svcs/
    
    kubectl describe ns $ns > $OUTPUT_DIR/$ns/namespace-$ns.log

    # ... get pods, and capture 'describe' & latest logs
    for pod in $(kubectl get pods -n $ns --output=jsonpath={.items..metadata.name}); do
        echo " - $pod "
        LOGFILE=$OUTPUT_DIR/$ns/pods/$pod.log
        echo "STATUS $pod at $NOW" > $LOGFILE
        echo "------------------------------------------------" >> $LOGFILE
        kubectl describe pod -n $ns $pod >> $LOGFILE
        echo "------------------------------------------------" >> $LOGFILE
        echo "LOGS" >> $LOGFILE
        echo "------------------------------------------------" >> $LOGFILE
        kubectl logs -n $ns $pod --since=$LOG_DURATION >> $LOGFILE
        grep -i "ERROR" $LOGFILE
    done

    # ... get services, and capture 'describe'
    for svc in $(kubectl get svc -n $ns --output=jsonpath={.items..metadata.name}); do
        echo " - $svc "
        LOGFILE=$OUTPUT_DIR/$ns/svcs/$svc.log
        echo "STATUS $svc at $NOW" > $LOGFILE
        echo "------------------------------------------------" >> $LOGFILE
        kubectl describe svc -n $ns $svc >> $LOGFILE
        grep -i "ERROR" $LOGFILE
    done

    # ... other elements?

done

# archive everything up
tar -czf $OUTPUT_DIR.tar.gz $OUTPUT_DIR

echo "Debug logs saved to $OUTPUT_DIR.tar.gz "

# Additional, but would need sudo:
# sudo iptables-save > $OUTPUT_DIR/info.txt
# ...any other debug info that should be included?
