#!/bin/bash
##########################################################################################
# Update IP address of a machine running the Davra platform
# 
# Must be run as sudo
# If an argument is supplied, we're assuming that the new IP to use
#   otherwise, try determine the new IP from the interfaces config
#   otherwise, try ip route (may not be available if offline)
# If no neew & valid IP, then exit
# If new IP is the same as the old IP, then exit
# 
# If there is a different IP, then begin:
#
#
#
HOME=$(pwd)
cd $(dirname $0)

source ../utils.sh

# check for new IP, supplied as an argument, from the network interfaces file, or from IP route
if [ ! -z "$1" ]; then
    NEW_IP=$1
else
    NEW_IP=`cat /etc/network/interfaces | grep address | sed -n '/address/{s/.*address[[:space:]]*\([^ ]*\).*/\1/p;q}'`
    if [ -z "$NEW_IP" ]; then
        NEW_IP=`ip route get 8.8.8.8 | sed -n '/src/{s/.*src *\([^ ]*\).*/\1/p;q}'`
    fi
fi

#root only
if [ "$(id -u)" != "0" ]; then
    echo "This script must be run as root" 1>&2
    exit 1
fi

if [ -z "$NEW_IP" ]; then
    echo "You must supply a new IP address to use" 1>&2
    exit 1
fi

function valid_ip {
    local  ip=$1
    local  stat=1
    
    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
        && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}

if ! valid_ip $NEW_IP; then
    echo " '$NEW_IP' looks like an invalid IP" 1>&2
    exit 1
fi

# NOTE: this may not be a hard requirement, but is strongly suggested for on-prem
if ! grep static /etc/network/interfaces; then
    echo "/etc/network/interfaces must be set to use a static IP" 1>&2
    exit 1
fi

OLD_IP=`cat /etc/kubernetes/admin.conf | grep server: | sed -n '/https\:\/\//{s/.*https\:\/\/ *\([^:]*\).*/\1/p;q}'`

DOMAINNAME=$(hostname --fqdn)

if [ "$NEW_IP" == "$OLD_IP" ]; then
    echo "IP hasn't changed for $DOMAINNAME,  $OLD_IP == $NEW_IP "
    exit 1
fi

echo "$DOMAINNAME IP was: $OLD_IP but changing to $NEW_IP - updating server..."


echo "Updating IPs in /etc/kubernetes/...  "
find /etc/kubernetes/ -type f -exec sed -i "s/$OLD_IP/$NEW_IP/g" {} + 

# If previous config has wrong subnet, fix that ...
sed "s/--service-cluster-ip-range=10.96.0.0/--service-cluster-ip-range=10.3.0.0/" /etc/kubernetes/manifests/kube-apiserver.yaml >/dev/null  2>&1

echo "Old API server Cert applies to:"
openssl x509 -in /etc/kubernetes/pki/apiserver.crt -text -noout | grep DNS

cp /etc/kubernetes/pki/apiserver.crt /tmp/
cp /etc/kubernetes/pki/apiserver.key /tmp/


cat >/tmp/csr.conf <<EOL
[ req ]
default_bits = 2048
prompt = no
default_md = sha256
req_extensions = req_ext
distinguished_name = dn

[ dn ]
CN = $NEW_IP

[ req_ext ]
subjectAltName = @alt_names

[ alt_names ]
DNS.1 = $DOMAINNAME
DNS.2 = kubernetes
DNS.3 = kubernetes.default
DNS.4 = kubernetes.default.svc
DNS.5 = kubernetes.default.svc.cluster.local
IP.2 = 10.3.0.1
IP.1 = $NEW_IP

[ v3_ext ]
authorityKeyIdentifier=keyid,issuer:always
basicConstraints=CA:FALSE
keyUsage=keyEncipherment,dataEncipherment
extendedKeyUsage=serverAuth,clientAuth
subjectAltName=@alt_names
EOL


echo "generate the signing request"
openssl req -new -key /etc/kubernetes/pki/apiserver.key -out /tmp/apiserver.csr -config /tmp/csr.conf

echo "use that new csr to generate the cert"
openssl x509 -req -in /tmp/apiserver.csr -CA /etc/kubernetes/pki/ca.crt -CAkey /etc/kubernetes/pki/ca.key -CAcreateserial -out /etc/kubernetes/pki/apiserver.crt -days 10000 -extensions v3_ext -extfile /tmp/csr.conf


echo "New API server Cert applies to:"
openssl x509 -in /etc/kubernetes/pki/apiserver.crt -text -noout | grep DNS

echo "--"
echo "etcd-peer:"
echo "old cert applies to:"
openssl x509 -in /etc/kubernetes/pki/etcd/peer.crt -text -noout | grep DNS

cat >/tmp/etcd-csr.conf <<EOL
[req]
default_bits = 2048
prompt = no
default_md = sha256
req_extensions = req_ext
distinguished_name = dn

[ dn ]
CN = $DOMAINNAME

[ alt_names ]
DNS.1 = $DOMAINNAME
DNS.2 = localhost
IP.1 = $NEW_IP
IP.2 = 127.0.0.1
IP.3 = 0:0:0:0:0:0:0:1

[ req_ext ]
keyUsage=keyEncipherment,dataEncipherment
extendedKeyUsage=serverAuth,clientAuth
subjectAltName=@alt_names
EOL


echo "generate the signing request"
openssl req -new -key /etc/kubernetes/pki/etcd/peer.key -out /tmp/etcd-peer.csr -config /tmp/etcd-csr.conf

echo "use that new csr to generate the cert"
openssl x509 -req -in /tmp/etcd-peer.csr -CA /etc/kubernetes/pki/etcd/ca.crt -CAkey /etc/kubernetes/pki/etcd/ca.key -CAcreateserial -out /etc/kubernetes/pki/etcd/peer.crt -days 10000 -extensions req_ext -extfile /tmp/etcd-csr.conf

echo "new cert applies to:"
openssl x509 -in /etc/kubernetes/pki/etcd/peer.crt -text -noout | grep DNS

echo "Updating conf files:"
API_SERVER_CERT=$(cat /etc/kubernetes/pki/apiserver.crt | base64 -w0)
API_SERVER_KEY=$(cat /etc/kubernetes/pki/apiserver.key | base64 -w0)

sed -i "/client-certificate/c\    client-certificate-data: $API_SERVER_CERT" /etc/kubernetes/admin.conf
sed -i "/client-key/c\    client-key-data: $API_SERVER_KEY" /etc/kubernetes/admin.conf

echo "Backup/remove the kubelet service keys"
mv /var/lib/kubelet/pki/ /tmp/kubelet-pki/

echo "Update user & root configs:"
cp /etc/kubernetes/admin.conf /root/.kube/config
cp /etc/kubernetes/admin.conf /home/$SUDO_USER/.kube/config

echo "Reload & restart the services"
systemctl daemon-reload
systemctl stop kubelet
systemctl restart docker
systemctl start kubelet

waitForKubernetes


cat << EOL
After kube comes back up, you'll need to manually edit 2 config maps:
In the first, kubeadm-config, find the old IP and change to the new one:
    ...
  ClusterStatus: |
    apiEndpoints:
      debian:
        advertiseAddress: ...OLD_IP...
        bindPort: 6443
    '''
EOL
read -p "Press any key to continue" answer
KUBE_EDITOR="nano" kubectl -n kube-system edit cm kubeadm-config


cat << EOL
Ok, now a similar update for the kube-proxy
Find this bit and update to the new ip:
    ...
  kubeconfig.conf: |-
    apiVersion: v1
    kind: Config
    clusters:
    - cluster:
        certificate-authority: /var/run/secrets/kubernetes.io/serviceaccount/ca.crt
        server: https:// .... OLD_IP ... :6443
      name: default
    ...
EOL
read -p "Press any key to continue" answer
KUBE_EDITOR="nano" kubectl -n kube-system edit cm kube-proxy


cat << EOL
And finally, ensure the nameservers in network/interfaces and resolv.conf match
 and aren't localhost, 127.0.0.1, or other 'self'.
Current details:
EOL
cat /etc/network/interfaces | grep nameserver
echo "The next editor will open resolv.conf, if needed update the nameserver to suit."
read -p "Press any key to continue" answer
nano /etc/resolv.conf


cat << EOL
OK done, pending a reboot.

NOTE: after rebooting, the system will take longer than normal to come back up.
If you watch kubectl get pods --all-namespaces -o wide you will see plenty of errors and crash loop back offs.
This is to be expected, and as ever with k8s, you just need to be patient - all should resolve in time.
If there are remaining issues after half an hour, then run the ./tools/trouble.sh and send the results to the
poor sod who will have to review them: karl.chadwick@davra.io

EOL

