#!/bin/bash

TIMEOUT_IN_MINUTES=30
TIMEOUT_IN_SECONDS=`expr $TIMEOUT_IN_MINUTES \* 60`
TIMEOUT_IN_LOOPS=`expr $TIMEOUT_IN_SECONDS / 3`
NOT_TIMEOUT=`expr $TIMEOUT_IN_LOOPS + 100`
REPEATEDLY_UP=10

function doThrottle {
    echo "Rate-limiting api calls"
    sleep 5
}

function doPV {
    kubectl get pv "$1-pv" > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating PV $1"
        kubectl create -f "$1-pv.yaml"
    else
        echo "Replacing PV $1"
        kubectl replace -f "$1-pv.yaml"
    fi
}

function doPVC {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get pvc "$1-pvc" -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating PVC $NAMESPACE.$1"
        kubectl create -f "$1-pvc.yaml" -n $NAMESPACE
    fi
}

function doDeployment {
    echo ""
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get deployment $1 -n $NAMESPACE > /dev/null 2>&1
    DEPLOYMENT=$?

    MS_NAME_WITHOUT_PREFIX=`echo $1 | sed 's/ruban-ms-//g'`

    kubectl get deployment $MS_NAME_WITHOUT_PREFIX -n $NAMESPACE > /dev/null 2>&1
    DEPLOYMENT_WITHOUT_PREFIX=$?

    if [ $DEPLOYMENT -eq 0  ] || [ $DEPLOYMENT_WITHOUT_PREFIX -eq 0 ] ; then
        # If deployment exists, ensure we preserved any custom resource requests & limits
        echo "Replacing deployment $NAMESPACE.$1"

        UTILS_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null && pwd )"

        if [ "$1" = "ruban" ]; then
            POD=$(kubectl get pod -n $NAMESPACE | grep $1 | head -1 | grep -v ruban-ms | awk '{print $1}')
        else
            POD=$(kubectl get pod -n $NAMESPACE | grep $1 | head -1 | awk '{print $1}')
        fi
        kubectl get pod $POD -n $NAMESPACE -o json > $UTILS_DIR/scripts/pod.json # Used by container below

        # NOTE: add a final param such as "npm start $1 revert" to remove all resource clauses from all pod deployments - use with care!
        CMD="docker run --network host -v $UTILS_DIR/scripts:/workspace -v `pwd`:/yamls node:8 bash -c 'cd /workspace; npm update; npm start $1'"
        eval $CMD
        rm $UTILS_DIR/scripts/pod.json

        #preserve replicas
        REPLICAS=`kubectl get deployment $1 -n $NAMESPACE -o yaml | grep replicas | tail -1 | awk -F ':' '{print $2}' | tr -d ' '`
        echo "Found $REPLICAS of deployment $1"
        sed s/'replicas: 1'/"replicas: $REPLICAS"/g "$1-deployment.yaml" > "tmp-$1-deployment.yaml"

        kubectl replace -f "tmp-$1-deployment.yaml" -n $NAMESPACE
        #rm -f "tmp-$1-deployment.yaml"
    else
        echo "Creating deployment $NAMESPACE.$1"
        kubectl create -f "$1-deployment.yaml" -n $NAMESPACE
    fi
}

function doDeploymentOld {
    echo ""
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get deployment $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -eq 0 ] ; then
        REPLICAS=`kubectl get deployment $1 -n $NAMESPACE | tail -n+2 | awk '{print $2}'`
        if [ $REPLICAS -lt 1 ] ; then
            REPLICAS=1
        fi

        echo "Scaling-down deployment $NAMESPACE.$1"
        scaleDeploymentOld $1 $NAMESPACE 0

        # If 3rd param true then wait for scale down
        if [ $# -gt 2 ] ; then

            cnt=0
            while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
            do
                kubectl get pod -n $NAMESPACE | grep $1-[0-9]*-[0-9a-zA-Z]*  > /dev/null 2>&1
                if [ $? -ne 0 ] ; then
                    cnt=`expr $NOT_TIMEOUT - 1`
                fi
                sleep 3
                printf "-"
                cnt=`expr $cnt + 1`
            done

            if [ $cnt -ne $NOT_TIMEOUT ] ; then
                echo "TIMEOUT! Deployment $NAMESPACE.$1 has not scaled-down"
                exit 1
            fi
        fi

        echo "Deployment scaled-down to 0 replicas"

        echo "Replacing deployment $NAMESPACE.$1"
        upsertDeploymentOld replace "$1-deployment.yaml" $NAMESPACE
        echo "Setting replicas of deployment $NAMESPACE.$1 to $REPLICAS"
        scaleDeploymentOld $1 $NAMESPACE $REPLICAS
    else
        echo "Creating deployment $NAMESPACE.$1"
        upsertDeploymentOld create "$1-deployment.yaml" $NAMESPACE
    fi
}

function waitOnDeployment {

    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi


    cntRepeatedlyUp=0
    while [ $cntRepeatedlyUp -lt $REPEATEDLY_UP ]
    do
        cntTimeOut=0
        while [ $cntTimeOut -lt $TIMEOUT_IN_LOOPS ]
        do
            kubectl get pod -n $NAMESPACE -l name=$1  | awk '{print $3}' | grep "Running"
            if [ $? -eq 0 ] ; then
                # Running
                cntTimeOut=`expr $NOT_TIMEOUT - 1`
            fi
            sleep 3
            printf "-"
            cntTimeOut=`expr $cntTimeOut + 1`
        done

        if [ $cntTimeOut -ne $NOT_TIMEOUT ] ; then
            echo "TIMEOUT! Deployment $NAMESPACE.$1 has not scaled-down"
            exit 1
        fi
        sleep 3
        printf "*"
        cntRepeatedlyUp=`expr $cntRepeatedlyUp + 1`
    done
}

function waitOnPodOld {

    echo "Wait on pod args: $0 $1 $2 $3 $4"


    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    echo "Waiting on pod $1 in namespace $NAMESPACE"

    cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl get pod $1 -n $NAMESPACE -o jsonpath='{range .status.containerStatuses[*]}{.ready}{"\n"}{end}' | grep  "true"
        if [ $? -eq 0 ] ; then
            cnt=`expr $NOT_TIMEOUT - 1`
        fi
        sleep 3
        printf "-"
        cnt=`expr $cnt + 1`
    done

    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! Pod $NAMESPACE.$1 has come up"
        exit 1
    fi

}

function waitOnPod {

    echo "Wait on pod args: $0 $1 $2 $3 $4"


    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    CONDITION="1/1"
    if [ $# -gt 2 ] ; then
        CONDITION=$3
    fi

    echo "Waiting on pod $1 in namespace $NAMESPACE, condition: $CONDITION"

    cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl get pod $1 -n $NAMESPACE  | grep  $CONDITION
        if [ $? -eq 0 ] ; then
            cnt=`expr $NOT_TIMEOUT - 1`
        fi
        sleep 3
        printf "-"
        cnt=`expr $cnt + 1`
    done

    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! Pod $NAMESPACE.$1 has not come up"
        exit 1
    fi

}

#
# Calls an endpoint over and over until a positive response is reached
# $1 the url to hit
# $2 the Bearer token to use
#
function waitOnEndpoint {

    echo "Wait on endpoint args: $0 $1 $2"


    echo "Waiting on endpoint $1"

    PASS_THRESHOLD=10
    cnt=0
	pass_cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        curl -f -s -k -H "Authorization: Bearer $2" $1 > /dev/null
        if [ $? -eq 0 ] ; then
            pass_cnt=`expr $pass_cnt + 1`
            if [ $pass_cnt -ge $PASS_THRESHOLD ] ; then
	            cnt=`expr $NOT_TIMEOUT - 1`
	        fi
	    else
	        #echo "Resetting pass counter to zero"
	        pass_cnt=0
	        printf "!"
	    fi
        sleep 3
        printf "-"
        cnt=`expr $cnt + 1`
    done

    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! Endpoint $1 has not come up"
        exit 1
    fi

}


function scaleDeploymentOld {
    count=0
    while [ $count -lt 5 ]
    do
        kubectl scale deployment $1 -n $2 --replicas=$3

        if [ $? -eq 0 ] ; then
            count=`expr 5`
        else
            count=`expr $count + 1`
            backoff=`expr $count * 5`
            echo "Scale operation failed retrying in $backoff seconds"
            sleep $backoff
        fi
    done

}

function upsertDeploymentOld {
    count=0
    while [ $count -lt 5 ]
    do
        kubectl $1 -f $2 -n $3

        if [ $? -eq 0 ] ; then
            count=`expr 5`
        else
            count=`expr $count + 1`
            backoff=`expr $count * 5`
            echo "Operation failed retrying in $backoff seconds"
            sleep $backoff
        fi
    done
}

function doService {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get service $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating service $NAMESPACE.$1"
        kubectl create -f "$1-service.yaml" -n $NAMESPACE
    else
        # If 3rd param true then re-create service (replace not possible)
        if [ $# -eq 3 ] ; then
            echo "Clobbering service $NAMESPACE.$1"
            kubectl delete service $1 -n $NAMESPACE
            kubectl create -f "$1-service.yaml" -n $NAMESPACE
        fi
    fi
}

function doNamespace {
    kubectl get namespace $1 > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating namespace $1"
        kubectl create -f "$1-namespace.yaml"
    fi
}

function doStorageclass {
    kubectl get storageclass $1 > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating storageclass $1"
        kubectl create -f "$1-storageclass.yaml"
    fi
}

function doConfigmap {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get configmap $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating configmap $NAMESPACE.$1"
        kubectl create -f "$1-configmap.yaml" -n $NAMESPACE
    else
        echo "NOTE: CONFIGMAP $1 ALREADY EXISTS. THIS WILL NOT BE UPDATED."
    fi
}

function doStatefulset {
    kubectl get statefulset $1 > /dev/null 2>&1
    if [ $? -eq 0 ] ; then
        echo "Warning you have to update statefulset: $1 by hand (with kubectl replace -f xxx-statefulset.yaml) until we can script the container image updates"
    else
        echo "Creating statefulset $1"
        kubectl create -f "$1-statefulset.yaml"
    fi
}

function doSecret {
    kubectl get secret $1 > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating secret $1"
        kubectl create -f "$1-secret.yaml"
    else
        echo "Replacing secret $1"
        kubectl replace -f "$1-secret.yaml"
    fi
}

function doIngress {
    
    # delete record if it's the beta ingress
    kubectl get ing $1 | grep v1beta1 > /dev/null 2>&1 
    if [ $? -eq 0 ] ; then
        echo "Deleting beta ingress record $1"
        kubectl delete ing $1
    fi
    
    kubectl get ing $1 > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating ingress $1"
        kubectl create -f "$1-ingress.yaml"
    else
        echo "Replacing ingress $1"
        kubectl apply -f "$1-ingress.yaml"
    fi
}

function doDaemonset {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get ds $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating daemonset $NAMESPACE.$1"
        kubectl create -f "$1-daemonset.yaml" -n $NAMESPACE
    else
        echo "Replacing daemonset $NAMESPACE.$1"
        kubectl replace -f "$1-daemonset.yaml" -n $NAMESPACE
    fi
}

function doServiceaccount {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get sa $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating service account $NAMESPACE.$1"
        kubectl create -f "$1-serviceaccount.yaml" -n $NAMESPACE
    else
        echo "Replacing service account $NAMESPACE.$1"
        kubectl replace -f "$1-serviceaccount.yaml" -n $NAMESPACE
    fi
}

function doRole {
    kubectl get roles $1 > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating role $1"
        kubectl create -f "$1-role.yaml"
    else
        echo "Replacing role $1"
        kubectl replace -f "$1-role.yaml"
    fi
}

function doRolebinding {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get rolebindings $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating role binding $NAMESPACE.$1"
        kubectl create -f "$1-rolebinding.yaml" -n $NAMESPACE
    else
        echo "Replacing role binding $NAMESPACE.$1"
        kubectl replace -f "$1-rolebinding.yaml" -n $NAMESPACE
    fi
}

function doClusterrole {
    kubectl get clusterroles $1 > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating cluster role $1"
        kubectl create -f "$1-clusterrole.yaml"
    else
        echo "Replacing cluster role $1"
        kubectl replace -f "$1-clusterrole.yaml"
    fi
}

function doClusterrolebinding {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get clusterrolebindings $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating cluster role binding $NAMESPACE.$1"
        kubectl create -f "$1-clusterrolebinding.yaml" -n $NAMESPACE
    else
        echo "Replacing cluster role binding $NAMESPACE.$1"
        kubectl replace -f "$1-clusterrolebinding.yaml" -n $NAMESPACE
    fi
}

function doHpa {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get hpa $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating horizontal pod autoscaler $NAMESPACE.$1"
        kubectl create -f "$1-hpa.yaml" -n $NAMESPACE
    else
        echo "Replacing hpa $NAMESPACE.$1"
        kubectl replace -f "$1-hpa.yaml" -n $NAMESPACE
    fi
}

function doCronjob {
    NAMESPACE="default"
    if [ $# -gt 1 ] ; then
        NAMESPACE=$2
    fi

    kubectl get cronjobs $1 -n $NAMESPACE > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        echo "Creating cron job $NAMESPACE.$1"
        kubectl create -f "$1-cronjob.yaml" -n $NAMESPACE
    else
        echo "Replacing cron job $NAMESPACE.$1"
        kubectl replace -f "$1-cronjob.yaml" -n $NAMESPACE
    fi
}

function waitForKubernetes {
    echo "Waiting for kubernetes"
    cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl get namespace kube-system > /dev/null 2>&1
        if [ $? -eq 0 ] ; then
            cnt=`expr $NOT_TIMEOUT - 1`
        fi
        sleep 3
        printf "-"
        cnt=`expr $cnt + 1`
    done

    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! Kubernetes is not working properly, examine journalctl -u kubelet for more info"
        exit 1
    fi
}

function waitForNginx {
    echo "Waiting for nginx"
    cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl get service -n ingress-nginx ingress-nginx-controller > /dev/null 2>&1
        if [ $? -eq 0 ] ; then
            cnt=`expr $NOT_TIMEOUT - 1`
        fi
        sleep 3
        printf "-"
        cnt=`expr $cnt + 1`
    done

    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! Nginx ingress not found"
        exit 1
    fi
}

#
# Connects to the given URI via Mongo shell from within the MongoDB pod
# $1 MongoDB URI
#
function checkMongoURI {
    echo "Checking MongoURI"
    MONGODB_POD=$(kubectl get pod -l name=mongodb | grep mongodb | awk '{print $1}')
    if [ -z "$1" ]; then
        echo "You need to provide a URI to test"
        exit 1
    else
        kubectl exec $MONGODB_POD -- mongo $1 > /dev/null 2>&1
        if [ $? -eq 0 ]; then
            echo "Connected to MongoDB"
        else
            echo "Could not connect to MongoDB"
            exit 1
        fi
    fi
}

#
# Calls an endpoint over and over until a positive response is reached
# $1 the url to hit
# $2 the Bearer token to use
#
function waitOnMongo {

    echo "Waiting on mongo"

    PASS_THRESHOLD=3
    cnt=0
	pass_cnt=0
    while [ $cnt -lt $TIMEOUT_IN_LOOPS ]
    do
        kubectl get pods -l name=mongodb | awk '{print $3}' | grep "Running"
        if [ $? -eq 0 ] ; then
            pass_cnt=`expr $pass_cnt + 1`
            if [ $pass_cnt -ge $PASS_THRESHOLD ] ; then
	            cnt=`expr $NOT_TIMEOUT - 1`
	        fi
	    else
	        #echo "Resetting pass counter to zero"
	        pass_cnt=0
	        printf "!"
	    fi
        sleep 3
        printf "-"
        cnt=`expr $cnt + 1`
    done

    if [ $cnt -ne $NOT_TIMEOUT ] ; then
        echo "TIMEOUT! MongoDB has not come up"
        exit 1
    fi

}


export doPVC
export doDeployment
export doService
export doNamespace
export doStatefulset
export doStorageclass
export doIngress
export doDaemonset
export doServiceaccount
export doRole
export doRolebinding
export doClusterrole
export doClusterrolebinding
export doHpa
export waitForKubernetes
export checkMongoURI
export waitOnMongo